@@ -1,20 +1,24 @@
-# validated: 2017-12-27 TW f9bece2ffbf7 edu/wpi/first/wpilibj/ADXL345_I2C.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN ecfe95383cdf edu/wpi/first/wpilibj/ADXL345_I2C.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+import enum
+
+from typing import Tuple, Optional
 
 import hal
-
-from .interfaces import Accelerometer
 from .i2c import I2C
-from .sensorbase import SensorBase
+from .sendablebase import SendableBase
+from .interfaces import Accelerometer
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["ADXL345_I2C"]
 
-class ADXL345_I2C(SensorBase):
+
+class ADXL345_I2C(SendableBase):
     """
         ADXL345 accelerometer device via i2c
     """
@@ -38,23 +42,23 @@ class ADXL345_I2C(SensorBase):
 
     Range = Accelerometer.Range
 
-    class Axes:
+    class Axes(enum.IntEnum):
         kX = 0x00
         kY = 0x02
         kZ = 0x04
 
-    def __init__(self, port, range, address=None):
+    def __init__(
+        self, port: I2C.Port, range: Range, address: Optional[int] = None
+    ) -> None:
         """Constructor.
 
         :param port: The I2C port the accelerometer is attached to.
-        :type port: :class:`.I2C.Port`
         :param range: The range (+ or -) that the accelerometer will measure.
-        :type range: :class:`.ADXL345_I2C.Range`
         :param address: the I2C address of the accelerometer (0x1D or 0x53)
         """
         if address is None:
             address = self.kAddress
-        
+
         self.i2c = I2C(port, address)
 
         # Turn on the measurements
@@ -62,23 +66,23 @@ class ADXL345_I2C(SensorBase):
 
         self.setRange(range)
 
-        hal.report(hal.UsageReporting.kResourceType_ADXL345,
-                      hal.UsageReporting.kADXL345_I2C)
+        hal.report(
+            hal.UsageReporting.kResourceType_ADXL345, hal.UsageReporting.kADXL345_I2C
+        )
 
         self.setName("ADXL345_I2C", port)
 
-    def free(self):
-        self.i2c.free()
-        super().free()
+    def close(self) -> None:
+        self.i2c.close()
+        super().close()
 
     # Accelerometer interface
 
-    def setRange(self, range):
+    def setRange(self, range: Range) -> None:
         """Set the measuring range of the accelerometer.
 
         :param range: The maximum acceleration, positive or negative, that
                       the accelerometer will measure.
-        :type  range: :class:`ADXL345_I2C.Range`
         """
         if range == self.Range.k2G:
             value = 0
@@ -94,28 +98,28 @@ class ADXL345_I2C(SensorBase):
         # Specify the data format to read
         self.i2c.write(self.kDataFormatRegister, self.kDataFormat_FullRes | value)
 
-    def getX(self):
+    def getX(self) -> float:
         """Get the x axis acceleration
 
         :returns: The acceleration along the x axis in g-forces
         """
         return self.getAcceleration(self.Axes.kX)
 
-    def getY(self):
+    def getY(self) -> float:
         """Get the y axis acceleration
 
         :returns: The acceleration along the y axis in g-forces
         """
         return self.getAcceleration(self.Axes.kY)
 
-    def getZ(self):
+    def getZ(self) -> float:
         """Get the z axis acceleration
 
         :returns: The acceleration along the z axis in g-forces
         """
         return self.getAcceleration(self.Axes.kZ)
 
-    def getAcceleration(self, axis):
+    def getAcceleration(self, axis: Axes) -> float:
         """Get the acceleration of one axis in Gs.
 
         :param axis: The axis to read from.
@@ -126,7 +130,7 @@ class ADXL345_I2C(SensorBase):
         rawAccel = (data[1] << 8) | data[0]
         return rawAccel * self.kGsPerLSB
 
-    def getAccelerations(self):
+    def getAccelerations(self) -> Tuple[float, float, float]:
         """Get the acceleration of all axes in Gs.
 
         :returns: X,Y,Z tuple of acceleration measured on all axes of the
@@ -137,20 +141,21 @@ class ADXL345_I2C(SensorBase):
         # Sensor is little endian... swap bytes
         rawData = []
         for i in range(3):
-            rawData.append((data[i*2+1] << 8) | data[i*2])
+            rawData.append((data[i * 2 + 1] << 8) | data[i * 2])
 
-        return (rawData[0] * self.kGsPerLSB,
-                rawData[1] * self.kGsPerLSB,
-                rawData[2] * self.kGsPerLSB)
+        return (
+            rawData[0] * self.kGsPerLSB,
+            rawData[1] * self.kGsPerLSB,
+            rawData[2] * self.kGsPerLSB,
+        )
 
-
-    def _updateValues(self):
+    def _updateValues(self) -> None:
         data = self.getAccelerations()
         self._entryX.setDouble(data[0])
         self._entryY.setDouble(data[1])
         self._entryZ.setDouble(data[2])
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("3AxisAccelerometer")
         self._entryX = builder.getEntry("X")
         self._entryY = builder.getEntry("Y")
