@@ -1,26 +1,30 @@
-# validated: 2017-12-27 TW f9bece2ffbf7 edu/wpi/first/wpilibj/ADXL345_SPI.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN ecfe95383cdf edu/wpi/first/wpilibj/ADXL345_SPI.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+import enum
+from typing import Tuple
 
 import hal
 
 from .interfaces import Accelerometer
 from .spi import SPI
-from .sensorbase import SensorBase
+from .sendablebase import SendableBase
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["ADXL345_SPI"]
 
-class ADXL345_SPI(SensorBase):
+
+class ADXL345_SPI(SendableBase):
     """
         ADXL345 accelerometer device via spi
         
         .. not_implemented: init
     """
-    
+
     kPowerCtlRegister = 0x2D
     kDataFormatRegister = 0x31
     kDataRegister = 0x32
@@ -42,24 +46,22 @@ class ADXL345_SPI(SensorBase):
 
     Range = Accelerometer.Range
 
-    class Axes:
+    class Axes(enum.IntEnum):
         kX = 0x00
         kY = 0x02
         kZ = 0x04
 
-    def __init__(self, port, range):
+    def __init__(self, port: SPI.Port, range: Range) -> None:
         """Constructor. Use this when the device is the first/only device on
         the bus
 
         :param port: The SPI port that the accelerometer is connected to
-        :type port: :class:`.SPI.Port`
         :param range: The range (+ or -) that the accelerometer will measure.
-        :type range: :class:`.ADXL345_SPI.Range`
         """
         self.spi = SPI(port)
         self.spi.setClockRate(500000)
         self.spi.setMSBFirst()
-        self.spi.setSampleDataOnFalling()
+        self.spi.setSampleDataOnTrailingEdge()
         self.spi.setClockActiveLow()
         self.spi.setChipSelectActiveHigh()
 
@@ -68,23 +70,23 @@ class ADXL345_SPI(SensorBase):
 
         self.setRange(range)
 
-        hal.report(hal.UsageReporting.kResourceType_ADXL345,
-                      hal.UsageReporting.kADXL345_SPI)
+        hal.report(
+            hal.UsageReporting.kResourceType_ADXL345, hal.UsageReporting.kADXL345_SPI
+        )
 
         self.setName("ADXL345_SPI", port)
 
-    def free(self):
-        self.spi.free()
-        super().free()
+    def close(self) -> None:
+        self.spi.close()
+        super().close()
 
     # Accelerometer interface
 
-    def setRange(self, range):
+    def setRange(self, range: Range) -> None:
         """Set the measuring range of the accelerometer.
 
         :param range: The maximum acceleration, positive or negative, that
                       the accelerometer will measure.
-        :type  range: :class:`ADXL345_SPI.Range`
         """
         if range == self.Range.k2G:
             value = 0
@@ -97,44 +99,46 @@ class ADXL345_SPI(SensorBase):
         else:
             raise ValueError("Invalid range argument '%s'" % range)
 
-        self.spi.write([self.kDataFormatRegister,
-                        self.kDataFormat_FullRes | value])
+        self.spi.write([self.kDataFormatRegister, self.kDataFormat_FullRes | value])
 
-    def getX(self):
+    def getX(self) -> float:
         """Get the x axis acceleration
 
         :returns: The acceleration along the x axis in g-forces
         """
         return self.getAcceleration(self.Axes.kX)
 
-    def getY(self):
+    def getY(self) -> float:
         """Get the y axis acceleration
 
         :returns: The acceleration along the y axis in g-forces
         """
         return self.getAcceleration(self.Axes.kY)
 
-    def getZ(self):
+    def getZ(self) -> float:
         """Get the z axis acceleration
 
         :returns: The acceleration along the z axis in g-forces
         """
         return self.getAcceleration(self.Axes.kZ)
 
-    def getAcceleration(self, axis):
+    def getAcceleration(self, axis: Axes) -> float:
         """Get the acceleration of one axis in Gs.
 
         :param axis: The axis to read from.
         :returns: An object containing the acceleration measured on each axis of the ADXL345 in Gs.
         """
-        data = [(self.kAddress_Read | self.kAddress_MultiByte |
-                 self.kDataRegister) + axis, 0, 0]
+        data = [
+            (self.kAddress_Read | self.kAddress_MultiByte | self.kDataRegister) + axis,
+            0,
+            0,
+        ]
         data = self.spi.transaction(data)
         # Sensor is little endian... swap bytes
         rawAccel = (data[2] << 8) | data[1]
         return rawAccel * self.kGsPerLSB
 
-    def getAccelerations(self):
+    def getAccelerations(self) -> Tuple[float, float, float]:
         """Get the acceleration of all axes in Gs.
 
         :returns: X,Y,Z tuple of acceleration measured on all axes of the
@@ -142,27 +146,28 @@ class ADXL345_SPI(SensorBase):
         """
         # Select the data address.
         data = [0] * 7
-        data[0] = (self.kAddress_Read | self.kAddress_MultiByte |
-                   self.kDataRegister)
+        data[0] = self.kAddress_Read | self.kAddress_MultiByte | self.kDataRegister
         data = self.spi.transaction(data)
 
         # Sensor is little endian... swap bytes
         rawData = []
         for i in range(3):
-            rawData.append((data[i*2+2] << 8) | data[i*2+1])
+            rawData.append((data[i * 2 + 2] << 8) | data[i * 2 + 1])
 
-        return (rawData[0] * self.kGsPerLSB,
-                rawData[1] * self.kGsPerLSB,
-                rawData[2] * self.kGsPerLSB)
+        return (
+            rawData[0] * self.kGsPerLSB,
+            rawData[1] * self.kGsPerLSB,
+            rawData[2] * self.kGsPerLSB,
+        )
 
     # Live Window code, only does anything if live window is activated.
-    def _updateValues(self):
+    def _updateValues(self) -> None:
         data = self.getAccelerations()
         self._entryX.setDouble(data[0])
         self._entryY.setDouble(data[1])
         self._entryZ.setDouble(data[2])
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("3AxisAccelerometer")
         self._entryX = builder.getEntry("X")
         self._entryY = builder.getEntry("Y")
