@@ -1,10 +1,10 @@
-# validated: 2017-12-22 EN f9bece2ffbf7 edu/wpi/first/wpilibj/AnalogInput.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN ecfe95383cdf edu/wpi/first/wpilibj/AnalogInput.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2017. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 import weakref
@@ -13,15 +13,19 @@ from .accumulatorresult import AccumulatorResult
 from .interfaces import PIDSource
 from .livewindow import LiveWindow
 from .resource import Resource
-from .sensorbase import SensorBase
+from .sendablebase import SendableBase
+from .sensorutil import SensorUtil
+from .sendablebuilder import SendableBuilder
 from .timer import Timer
 
 __all__ = ["AnalogInput"]
 
-def _freeAnalogInput(port):
+
+def _freeAnalogInput(port: hal.AnalogInputHandle) -> None:
     hal.freeAnalogInputPort(port)
 
-class AnalogInput(SensorBase):
+
+class AnalogInput(SendableBase):
     """Analog input
 
     Each analog channel is read from hardware as a 12-bit number representing
@@ -39,19 +43,19 @@ class AnalogInput(SensorBase):
 
     kAccumulatorSlot = 1
     kAccumulatorChannels = (0, 1)
-    channels = Resource(SensorBase.kAnalogInputChannels)
-    
+    channels = Resource(SensorUtil.kAnalogInputChannels)
+
     PIDSourceType = PIDSource.PIDSourceType
 
-    def __init__(self, channel):
+    def __init__(self, channel: int) -> None:
         """Construct an analog channel.
 
         :param channel: The channel number to represent. 0-3 are on-board 4-7 are on the MXP port.
         """
-        
+
         super().__init__()
-        SensorBase.checkAnalogInputChannel(channel)
-        
+        hal.checkAnalogInputChannel(channel)
+
         self.channel = channel
         self.accumulatorOffset = 0
         self.pidSource = self.PIDSourceType.kDisplacement
@@ -59,21 +63,20 @@ class AnalogInput(SensorBase):
         port = hal.getPort(channel)
         self._port = hal.initializeAnalogInputPort(port)
 
-        hal.report(hal.UsageReporting.kResourceType_AnalogChannel,
-                      channel)
+        hal.report(hal.UsageReporting.kResourceType_AnalogChannel, channel)
         self.setName("AnalogInput", self.channel)
-        
+
         self.__finalizer = weakref.finalize(self, _freeAnalogInput, self._port)
-        
+
     @property
     def port(self):
         if not self.__finalizer.alive:
             return None
         return self._port
 
-    def free(self):
+    def close(self) -> None:
         """ Channel destructor """
-        super().free()
+        super().close()
         if self.channel is None:
             return
         AnalogInput.channels.free(self.channel)
@@ -81,7 +84,7 @@ class AnalogInput(SensorBase):
         self.channel = None
         self.accumulatorOffset = 0
 
-    def getValue(self):
+    def getValue(self) -> int:
         """Get a sample straight from this channel. The sample is a 12-bit
         value representing the 0V to 5V range of the A/D converter. The units
         are in A/D converter codes. Use :func:`getVoltage` to get the analog
@@ -91,7 +94,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogValue(self.port)
 
-    def getAverageValue(self):
+    def getAverageValue(self) -> float:
         """Get a sample from the output of the oversample and average engine
         for this channel. The sample is 12-bit + the bits configured in
         :func:`setOversampleBits`. The value configured in
@@ -106,7 +109,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogAverageValue(self.port)
 
-    def getVoltage(self):
+    def getVoltage(self) -> float:
         """Get a scaled sample straight from this channel. The value is scaled
         to units of Volts using the calibrated scaling data from
         :func:`getLSBWeight` and :func:`getOffset`.
@@ -115,7 +118,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogVoltage(self.port)
 
-    def getAverageVoltage(self):
+    def getAverageVoltage(self) -> float:
         """Get a scaled sample from the output of the oversample and average
         engine for this channel. The value is scaled to units of Volts using
         the calibrated scaling data from :func:`getLSBWeight` and
@@ -129,7 +132,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogAverageVoltage(self.port)
 
-    def getLSBWeight(self):
+    def getLSBWeight(self) -> float:
         """Get the factory scaling least significant bit weight constant. The
         least significant bit weight constant for the channel that was
         calibrated in manufacturing and stored in an eeprom.
@@ -140,7 +143,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogLSBWeight(self.port)
 
-    def getOffset(self):
+    def getOffset(self) -> int:
         """Get the factory scaling offset constant. The offset constant for the
         channel that was calibrated in manufacturing and stored in an eeprom.
 
@@ -150,14 +153,14 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogOffset(self.port)
 
-    def getChannel(self):
+    def getChannel(self) -> int:
         """Get the channel number.
 
         :returns: The channel number.
         """
         return self.channel
 
-    def setAverageBits(self, bits):
+    def setAverageBits(self, bits: int) -> None:
         """Set the number of averaging bits. This sets the number of
         averaging bits.  The actual number of averaged samples is 2^bits.
         The averaging is done automatically in the FPGA.
@@ -166,7 +169,7 @@ class AnalogInput(SensorBase):
         """
         hal.setAnalogAverageBits(self.port, bits)
 
-    def getAverageBits(self):
+    def getAverageBits(self) -> int:
         """Get the number of averaging bits. This gets the number of averaging
         bits from the FPGA. The actual number of averaged samples is 2^bits.
         The averaging is done automatically in the FPGA.
@@ -175,7 +178,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogAverageBits(self.port)
 
-    def setOversampleBits(self, bits):
+    def setOversampleBits(self, bits: int) -> None:
         """Set the number of oversample bits. This sets the number of
         oversample bits. The actual number of oversampled values is 2^bits.
         The oversampling is done automatically in the FPGA.
@@ -184,7 +187,7 @@ class AnalogInput(SensorBase):
         """
         hal.setAnalogOversampleBits(self.port, bits)
 
-    def getOversampleBits(self):
+    def getOversampleBits(self) -> int:
         """Get the number of oversample bits. This gets the number of
         oversample bits from the FPGA. The actual number of oversampled values
         is 2^bits.  The oversampling is done automatically in the FPGA.
@@ -193,18 +196,21 @@ class AnalogInput(SensorBase):
         """
         return hal.getAnalogOversampleBits(self.port)
 
-    def initAccumulator(self):
+    def initAccumulator(self) -> None:
         """Initialize the accumulator.
         """
         if not self.isAccumulatorChannel():
             raise IndexError(
-                    "Accumulators are only available on slot %d on channels %s"
-                    % (AnalogInput.kAccumulatorSlot,
-                       ",".join(str(c) for c in AnalogInput.kAccumulatorChannels)))
+                "Accumulators are only available on slot %d on channels %s"
+                % (
+                    AnalogInput.kAccumulatorSlot,
+                    ",".join(str(c) for c in AnalogInput.kAccumulatorChannels),
+                )
+            )
         self.accumulatorOffset = 0
         hal.initAccumulator(self.port)
 
-    def setAccumulatorInitialValue(self, initialValue):
+    def setAccumulatorInitialValue(self, initialValue: float) -> None:
         """Set an initial value for the accumulator.
 
         This will be added to all values returned to the user.
@@ -214,7 +220,7 @@ class AnalogInput(SensorBase):
         """
         self.accumulatorOffset = initialValue
 
-    def resetAccumulator(self):
+    def resetAccumulator(self) -> None:
         """Resets the accumulator to the initial value.
         """
         hal.resetAccumulator(self.port)
@@ -227,7 +233,7 @@ class AnalogInput(SensorBase):
         if not hal.HALIsSimulation():
             Timer.delay(sampleTime * overSamples * averageSamples)
 
-    def setAccumulatorCenter(self, center):
+    def setAccumulatorCenter(self, center: int) -> None:
         """Set the center value of the accumulator.
 
         The center value is subtracted from each A/D value before it is added
@@ -241,12 +247,12 @@ class AnalogInput(SensorBase):
         """
         hal.setAccumulatorCenter(self.port, center)
 
-    def setAccumulatorDeadband(self, deadband):
+    def setAccumulatorDeadband(self, deadband: int) -> None:
         """Set the accumulator's deadband.
         """
         hal.setAccumulatorDeadband(self.port, deadband)
 
-    def getAccumulatorValue(self):
+    def getAccumulatorValue(self) -> float:
         """Read the accumulated value.
 
         Read the value that has been accumulating. The accumulator
@@ -256,7 +262,7 @@ class AnalogInput(SensorBase):
         """
         return hal.getAccumulatorValue(self.port) + self.accumulatorOffset
 
-    def getAccumulatorCount(self):
+    def getAccumulatorCount(self) -> float:
         """Read the number of accumulated values.
 
         Read the count of the accumulated values since the last call to
@@ -280,7 +286,7 @@ class AnalogInput(SensorBase):
 
         return AccumulatorResult(value + self.accumulatorOffset, count)
 
-    def isAccumulatorChannel(self):
+    def isAccumulatorChannel(self) -> bool:
         """Is the channel attached to an accumulator.
 
         :returns: The analog channel is attached to an accumulator.
@@ -288,7 +294,7 @@ class AnalogInput(SensorBase):
         return self.channel in AnalogInput.kAccumulatorChannels
 
     @staticmethod
-    def setGlobalSampleRate(samplesPerSecond):
+    def setGlobalSampleRate(samplesPerSecond: float) -> None:
         """Set the sample rate per channel.
 
         This is a global setting for all channels.
@@ -300,7 +306,7 @@ class AnalogInput(SensorBase):
         hal.setAnalogSampleRate(float(samplesPerSecond))
 
     @staticmethod
-    def getGlobalSampleRate():
+    def getGlobalSampleRate() -> float:
         """Get the current sample rate.
 
         This assumes one entry in the scan list. This is a global setting for
@@ -309,22 +315,22 @@ class AnalogInput(SensorBase):
         :returns: Sample rate.
         """
         return hal.getAnalogSampleRate()
-    
-    def setPIDSourceType(self, pidSource):
+
+    def setPIDSourceType(self, pidSource: PIDSourceType) -> None:
         """:see: :meth:`.PIDSource.setPIDSourceType`"""
         self.pidSource = pidSource
-        
-    def getPIDSourceType(self):
+
+    def getPIDSourceType(self) -> PIDSourceType:
         """:see: :meth:`.PIDSource.getPIDSourceType`"""
         return self.pidSource
 
-    def pidGet(self):
+    def pidGet(self) -> float:
         """Get the average voltage for use with PIDController
 
         :returns: the average voltage
         """
         return self.getAverageVoltage()
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("Analog Input")
         builder.addDoubleProperty("Value", self.getAverageVoltage, None)
