@@ -1,4 +1,4 @@
-# validated: 2017-12-27 TW f9bece2ffbf7 edu/wpi/first/wpilibj/AnalogTrigger.java
+# validated: 2018-09-09 EN 0e9172f9a708 edu/wpi/first/wpilibj/AnalogTrigger.java
 # ----------------------------------------------------------------------------
 # Copyright (c) 2008-2017 FIRST. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -7,21 +7,23 @@
 # ----------------------------------------------------------------------------
 
 import weakref
+from typing import Union
 
 import hal
 from .analoginput import AnalogInput
 from .analogtriggeroutput import AnalogTriggerOutput
 from .resource import Resource
-from .sensorbase import SensorBase
+from .sendablebase import SendableBase
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["AnalogTrigger"]
 
 
-def _freeAnalogTrigger(port):
+def _freeAnalogTrigger(port: hal.AnalogTriggerHandle) -> None:
     hal.cleanAnalogTrigger(port)
 
 
-class AnalogTrigger(SensorBase):
+class AnalogTrigger(SendableBase):
     """
         Converts an analog signal into a digital signal
         
@@ -38,7 +40,7 @@ class AnalogTrigger(SensorBase):
 
     AnalogTriggerType = AnalogTriggerOutput.AnalogTriggerType
 
-    def __init__(self, channel):
+    def __init__(self, channel: Union[AnalogInput, int]) -> None:
         """Constructor for an analog trigger given a channel number or analog
         input.
 
@@ -55,31 +57,29 @@ class AnalogTrigger(SensorBase):
             self.analogInput = channel
 
         self._port, self.index = hal.initializeAnalogTrigger(self.analogInput.port)
-        self.__finalizer = \
-            weakref.finalize(self, _freeAnalogTrigger, self._port)
+        self.__finalizer = weakref.finalize(self, _freeAnalogTrigger, self._port)
 
         # Need this to free on unit test wpilib reset
         Resource._add_global_resource(self)
 
-        hal.report(hal.UsageReporting.kResourceType_AnalogTrigger,
-                   channel)
+        hal.report(hal.UsageReporting.kResourceType_AnalogTrigger, channel)
 
         self.setName("AnalogTrigger", self.analogInput.getChannel())
 
     @property
     def port(self):
         if not self.__finalizer.alive:
-            raise ValueError("Cannot use AnalogTrigger after free() has been called")
+            raise ValueError("Cannot use AnalogTrigger after close() has been called")
         return self._port
 
-    def free(self):
+    def close(self) -> None:
         """Release the resources used by this object"""
-        super().free()
+        super().close()
         self.__finalizer()
         if self.analogInput:
-            self.analogInput.free()
+            self.analogInput.close()
 
-    def setLimitsRaw(self, lower, upper):
+    def setLimitsRaw(self, lower: int, upper: int) -> None:
         """Set the upper and lower limits of the analog trigger. The limits are
         given in ADC codes. If oversampling is used, the units must be scaled
         appropriately.
@@ -92,7 +92,7 @@ class AnalogTrigger(SensorBase):
 
         hal.setAnalogTriggerLimitsRaw(self.port, lower, upper)
 
-    def setLimitsVoltage(self, lower, upper):
+    def setLimitsVoltage(self, lower: float, upper: float) -> None:
         """Set the upper and lower limits of the analog trigger. The limits are
         given as floating point voltage values.
 
@@ -104,7 +104,7 @@ class AnalogTrigger(SensorBase):
 
         hal.setAnalogTriggerLimitsVoltage(self.port, float(lower), float(upper))
 
-    def setAveraged(self, useAveragedValue):
+    def setAveraged(self, useAveragedValue: bool) -> None:
         """Configure the analog trigger to use the averaged vs. raw values. If
         the value is true, then the averaged value is selected for the analog
         trigger, otherwise the immediate value is used.
@@ -113,7 +113,7 @@ class AnalogTrigger(SensorBase):
         """
         hal.setAnalogTriggerAveraged(self.port, useAveragedValue)
 
-    def setFiltered(self, useFilteredValue):
+    def setFiltered(self, useFilteredValue: bool) -> None:
         """Configure the analog trigger to use a filtered value. The analog
         trigger will operate with a 3 point average rejection filter. This is
         designed to help with 360 degree pot applications for the period where
@@ -123,7 +123,7 @@ class AnalogTrigger(SensorBase):
         """
         hal.setAnalogTriggerFiltered(self.port, useFilteredValue)
 
-    def getIndex(self):
+    def getIndex(self) -> int:
         """Return the index of the analog trigger. This is the FPGA index of
         this analog trigger instance.
 
@@ -131,7 +131,7 @@ class AnalogTrigger(SensorBase):
         """
         return self.index
 
-    def getInWindow(self):
+    def getInWindow(self) -> bool:
         """Return the InWindow output of the analog trigger. True if the
         analog input is between the upper and lower limits.
 
@@ -139,7 +139,7 @@ class AnalogTrigger(SensorBase):
         """
         return hal.getAnalogTriggerInWindow(self.port)
 
-    def getTriggerState(self):
+    def getTriggerState(self) -> bool:
         """Return the TriggerState output of the analog trigger. True if above
         upper limit. False if below lower limit. If in Hysteresis, maintain
         previous state.
@@ -148,7 +148,7 @@ class AnalogTrigger(SensorBase):
         """
         return hal.getAnalogTriggerTriggerState(self.port)
 
-    def createOutput(self, type):
+    def createOutput(self, type: AnalogTriggerType) -> AnalogTriggerOutput:
         """Creates an :class:`.AnalogTriggerOutput` object. Gets an output object that
         can be used for routing. Caller is responsible for deleting the
         AnalogTriggerOutput object.
@@ -158,6 +158,6 @@ class AnalogTrigger(SensorBase):
         """
         return AnalogTriggerOutput(self, type)
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         if self.ownsAnalog:
             self.analogInput.initSendable(builder)
