@@ -1,18 +1,20 @@
-# validated: 2017-12-27 TW f9bece2ffbf7 edu/wpi/first/wpilibj/BuiltInAccelerometer.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN ecfe95383cdf edu/wpi/first/wpilibj/BuiltInAccelerometer.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2014-2016. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
 from .interfaces import Accelerometer
-from .sensorbase import SensorBase
+from .sendablebuilder import SendableBuilder
+from .sendablebase import SendableBase
 
 __all__ = ["BuiltInAccelerometer"]
 
-class BuiltInAccelerometer(SensorBase):
+
+class BuiltInAccelerometer(SendableBase):
     """Built-in accelerometer device
 
     This class allows access to the roboRIO's internal accelerometer.
@@ -20,30 +22,32 @@ class BuiltInAccelerometer(SensorBase):
 
     Range = Accelerometer.Range
 
-    def __init__(self, range=Accelerometer.Range.k8G):
+    def __init__(self, range: Range = Accelerometer.Range.k8G) -> None:
         """Constructor.
         
         :param range: The range the accelerometer will measure.  Defaults to
             +/-8g if unspecified.
-        :type  range: :class:`.Accelerometer.Range`
         """
         super().__init__()
         self.setRange(range)
         self.xEntry = None
         self.yEntry = None
         self.zEntry = None
-        hal.report(hal.UsageReporting.kResourceType_Accelerometer, 0, 0,
-                      "Built-in accelerometer")
+        hal.report(
+            hal.UsageReporting.kResourceType_Accelerometer,
+            0,
+            0,
+            "Built-in accelerometer",
+        )
         self.setName("BuiltInAccel", 0)
 
-    def setRange(self, range):
+    def setRange(self, range: Range) -> None:
         """Set the measuring range of the accelerometer.
 
         :param range: The maximum acceleration, positive or negative, that
                       the accelerometer will measure.
-        :type  range: :class:`BuiltInAccelerometer.Range`
         """
-        
+
         hal.setAccelerometerActive(False)
 
         if range == self.Range.k2G:
@@ -59,31 +63,28 @@ class BuiltInAccelerometer(SensorBase):
 
         hal.setAccelerometerActive(True)
 
-    def getX(self):
+    def getX(self) -> float:
         """
            :returns: The acceleration of the roboRIO along the X axis in
                      g-forces
-           :rtype: float
         """
         return hal.getAccelerometerX()
 
-    def getY(self):
+    def getY(self) -> float:
         """
            :returns: The acceleration of the roboRIO along the Y axis in
                      g-forces
-           :rtype: float
         """
         return hal.getAccelerometerY()
 
-    def getZ(self):
+    def getZ(self) -> float:
         """
            :returns: The acceleration of the roboRIO along the Z axis in
                      g-forces
-           :rtype: float
         """
         return hal.getAccelerometerZ()
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("3AxisAccelerometer")
         builder.addDoubleProperty("X", self.getX, None)
         builder.addDoubleProperty("Y", self.getY, None)
