@@ -1,4 +1,4 @@
-# validated: 2017-12-27 TW f9bece2ffbf7 edu/wpi/first/wpilibj/buttons/Trigger.java
+# validated: 2018-10-30 EN 0a2ab4f0d782 edu/wpi/first/wpilibj/buttons/Trigger.java
 # ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2017. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -6,9 +6,9 @@
 # the project.
 # ----------------------------------------------------------------------------
 
-
-from ..command.command import Command
+from ..command import command
 from ..sendablebase import SendableBase
+from ..sendablebuilder import SendableBuilder
 
 __all__ = ["Trigger"]
 
@@ -42,25 +42,26 @@ class Trigger(SendableBase):
         """
         return self.get() or getattr(self, "sendablePressed", False)
 
-    def whenActive(self, command: Command) -> None:
+    def whenActive(self, command: "command.Command") -> None:
         """Starts the given command whenever the trigger just becomes active.
 
         :param command: the command to start
         """
 
-        def execute():
-            if self.grab():
-                if not execute.pressedLast:
-                    execute.pressedLast = True
-                    command.start()
-            else:
-                execute.pressedLast = False
+        def execute() -> None:
+            pressed = self.grab()
+
+            if not execute.pressedLast and pressed:
+                command.start()
+
+            execute.pressedLast = pressed
 
         execute.pressedLast = self.grab()
         from ..command import Scheduler
+
         Scheduler.getInstance().addButton(execute)
 
-    def whileActive(self, command):
+    def whileActive(self, command: "command.Command") -> None:
         """Constantly starts the given command while the button is held.
 
         :meth:`Command.start` will be called repeatedly while the trigger is
@@ -69,74 +70,79 @@ class Trigger(SendableBase):
         :param command: the command to start
         """
 
-        def execute():
-            if self.grab():
-                execute.pressedLast = True
+        def execute() -> None:
+            pressed = self.grab()
+
+            if pressed:
                 command.start()
-            else:
-                if execute.pressedLast:
-                    execute.pressedLast = False
-                    command.cancel()
+            elif execute.pressedLast and not pressed:
+                command.cancel()
+
+            execute.pressedLast = pressed
 
         execute.pressedLast = self.grab()
         from ..command import Scheduler
+
         Scheduler.getInstance().addButton(execute)
 
-    def whenInactive(self, command: Command):
+    def whenInactive(self, command: "command.Command") -> None:
         """Starts the command when the trigger becomes inactive.
 
         :param command: the command to start
         """
 
-        def execute():
-            if self.grab():
-                execute.pressedLast = True
-            else:
-                if execute.pressedLast:
-                    execute.pressedLast = False
-                    command.start()
+        def execute() -> None:
+            pressed = self.grab()
+
+            if execute.pressedLast and not pressed:
+                command.start()
+
+            execute.pressedLast = pressed
 
         execute.pressedLast = self.grab()
         from ..command import Scheduler
+
         Scheduler.getInstance().addButton(execute)
 
-    def toggleWhenActive(self, command: Command):
+    def toggleWhenActive(self, command: "command.Command") -> None:
         """Toggles a command when the trigger becomes active.
 
         :param command: the command to toggle
         """
 
-        def execute():
-            if self.grab():
-                if not execute.pressedLast:
-                    execute.pressedLast = True
-                    if command.isRunning():
-                        command.cancel()
-                    else:
-                        command.start()
-            else:
-                execute.pressedLast = False
+        def execute() -> None:
+            pressed = self.grab()
+
+            if not execute.pressedLast and pressed:
+                if command.isRunning():
+                    command.cancel()
+                else:
+                    command.start()
+
+            execute.pressedLast = pressed
 
         execute.pressedLast = self.grab()
         from ..command import Scheduler
+
         Scheduler.getInstance().addButton(execute)
 
-    def cancelWhenActive(self, command: Command) -> None:
+    def cancelWhenActive(self, command: "command.Command") -> None:
         """Cancels a command when the trigger becomes active.
 
         :param command: the command to cancel
         """
 
-        def execute():
-            if self.grab():
-                if not execute.pressedLast:
-                    execute.pressedLast = True
-                    command.cancel()
-            else:
-                execute.pressedLast = False
+        def execute() -> None:
+            pressed = self.grab()
+
+            if not execute.pressedLast and pressed:
+                command.cancel()
+
+            execute.pressedLast = pressed
 
         execute.pressedLast = self.grab()
         from ..command import Scheduler
+
         Scheduler.getInstance().addButton(execute)
 
     def _safeState(self) -> None:
@@ -145,7 +151,7 @@ class Trigger(SendableBase):
     def _setPressed(self, value: bool) -> None:
         self.sendablePressed = value
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("Button")
         builder.setSafeState(self._safeState)
         builder.addBooleanProperty("pressed", self.grab, self._setPressed)
