@@ -1,10 +1,10 @@
-# validated: 2018-01-27 DS e4e1eab4131d edu/wpi/first/wpilibj/command/Command.java
-#----------------------------------------------------------------------------
+# validated: 2018-10-30 EN 0b113ad9ce93 edu/wpi/first/wpilibj/command/Command.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2016. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 from .scheduler import Scheduler
 from ..robotstate import RobotState
@@ -16,6 +16,7 @@ import threading
 
 __all__ = ["Command"]
 
+
 class Command(SendableBase):
     """The Command class is at the very core of the entire command framework.
     Every command can be started with a call to start().
@@ -41,13 +42,14 @@ class Command(SendableBase):
     .. seealso:: :class:`.Subsystem`, :class:`.CommandGroup`
     """
 
-    def __init__(self, name=None, timeout=None):
+    def __init__(self, name=None, timeout=None, subsystem=None):
         """Creates a new command.
         
         :param name: The name for this command; if unspecified or None,
                      The name of this command will be set to its class name.
         :param timeout: The time (in seconds) before this command "times out".
                         Default is no timeout.  See isTimedOut().
+        :param subsystem: The subsystem that this command requires
         """
         super().__init__(False)
         self.mutex = threading.RLock()
@@ -87,6 +89,9 @@ class Command(SendableBase):
         # The CommandGroup this is in
         self.parent = None
 
+        if subsystem is not None:
+            self.requires(subsystem)
+
     def setTimeout(self, seconds):
         """Sets the timeout of this command.
         
@@ -151,7 +156,11 @@ class Command(SendableBase):
         :returns: whether or not the command should stay within the Scheduler.
         """
         with self.mutex:
-            if not self.runWhenDisabled and self.parent is None and RobotState.isDisabled():
+            if (
+                not self.runWhenDisabled
+                and self.parent is None
+                and RobotState.isDisabled()
+            ):
                 self.cancel()
             if self.isCanceled():
                 return False
@@ -245,8 +254,7 @@ class Command(SendableBase):
         :returns: whether the time has expired
         """
         with self.mutex:
-            return (self.timeout != -1 and
-                    self.timeSinceInitialized() >= self.timeout)
+            return self.timeout != -1 and self.timeSinceInitialized() >= self.timeout
 
     def getRequirements(self):
         """Returns the requirements (as a set of Subsystems) of this command
@@ -268,10 +276,12 @@ class Command(SendableBase):
         """
         with self.mutex:
             if self.parent is not None:
-                raise ValueError("Can not give command to a command group after already being put in a command group")
+                raise ValueError(
+                    "Can not give command to a command group after already being put in a command group"
+                )
             self.lockChanges()
             self.parent = parent
-            
+
     def isParented(self):
         """
         Returns whether the command has a parent.
@@ -297,7 +307,9 @@ class Command(SendableBase):
         with self.mutex:
             self.lockChanges()
             if self.parent is not None:
-                raise ValueError("Can not start a command that is a part of a command group")
+                raise ValueError(
+                    "Can not start a command that is a part of a command group"
+                )
             Scheduler.getInstance().add(self)
             self.completed = False
 
@@ -358,7 +370,7 @@ class Command(SendableBase):
         """
         with self.mutex:
             return self.canceled
-    
+
     def isCompleted(self) -> bool:
         """Whether or not this command has completed running.
         
