@@ -1,15 +1,17 @@
 # validated: 2017-10-03 EN e1195e8b9dab edu/wpi/first/wpilibj/command/WaitCommand.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2016. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+from typing import Optional
 
 from .timedcommand import TimedCommand
 
 __all__ = ["WaitCommand"]
 
+
 class WaitCommand(TimedCommand):
     """A WaitCommand will wait for a certain amount of time before finishing.
     It is useful if you want a :class:`.CommandGroup` to pause for a moment.
@@ -17,7 +19,7 @@ class WaitCommand(TimedCommand):
     .. seealso:: :class:`.CommandGroup`
     """
 
-    def __init__(self, timeout, name=None):
+    def __init__(self, timeout: float, name: Optional[str] = None) -> None:
         """Instantiates a WaitCommand with the given timeout.
         
         :param timeout: the time the command takes to run (seconds)
