@@ -1,4 +1,4 @@
-# validated: 2017-12-23 EN f9bece2ffbf7 edu/wpi/first/wpilibj/DigitalInput.java
+# validated: 2018-09-18 EN ecfe95383cdf edu/wpi/first/wpilibj/DigitalInput.java
 # ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -9,6 +9,8 @@
 import hal
 
 from .digitalsource import DigitalSource
+from .sensorutil import SensorUtil
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["DigitalInput"]
 
@@ -23,71 +25,67 @@ class DigitalInput(DigitalSource):
     etc. that aren't implemented anywhere else.
     """
 
-    def __init__(self, channel):
+    def __init__(self, channel: int) -> None:
         """Create an instance of a Digital Input class. Creates a digital
         input given a channel.
 
         :param channel: the DIO channel for the digital input. 0-9 are on-board, 10-25 are on the MXP
-        :type  channel: int
         """
 
         super().__init__()
-        self.checkDigitalChannel(channel)
+        SensorUtil.checkDigitalChannel(channel)
         self.channel = channel
 
         self.handle = hal.initializeDIOPort(hal.getPort(channel), True)
 
-        hal.report(hal.UsageReporting.kResourceType_DigitalInput,
-                   channel)
+        hal.report(hal.UsageReporting.kResourceType_DigitalInput, channel)
         self.setName("DigitalInput", channel)
 
-    def free(self):
-        super().free()
+    def close(self) -> None:
+        super().close()
+
         if self.interrupt:
             self.cancelInterrupts()
 
         hal.freeDIOPort(self.handle)
+        self.handle = 0
 
-    def get(self):
+    def get(self) -> bool:
         """Get the value from a digital input channel. Retrieve the value of
         a single digital input channel from the FPGA.
 
         :returns: the state of the digital input
-        :rtype: bool
         """
         return hal.getDIO(self.handle)
 
-    def getChannel(self):
+    def getChannel(self) -> int:
         """Get the channel of the digital input.
 
         :returns: The GPIO channel number that this object represents.
-        :rtype: int
         """
         return self.channel
 
-    def getAnalogTriggerTypeForRouting(self):
+    def getAnalogTriggerTypeForRouting(self) -> int:
         """Get the analog trigger type.
 
         :returns: false
-        :rtype: int
         """
         return 0
 
-    def isAnalogTrigger(self):
+    def isAnalogTrigger(self) -> bool:
         """Is this an analog trigger.
 
         :returns: true if this is an analog trigger
-        :rtype: bool
         """
         return False
 
-    def getPortHandleForRouting(self):
+    def getPortHandleForRouting(self) -> hal.DigitalHandle:
         """Get the HAL Port Handle.
 
         :return: The HAL Handle to the specified source
         """
         return self.handle
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("Digital Input")
         builder.addBooleanProperty("Value", self.get, None)
