@@ -1,27 +1,29 @@
-# validated: 2017-12-12 EN f9bece2ffbf7 edu/wpi/first/wpilibj/DoubleSolenoid.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN 0614913f1abb edu/wpi/first/wpilibj/DoubleSolenoid.java
+# ----------------------------------------------------------------------------
 # Copyright (c) 2008-2017 FIRST. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 import enum
 import warnings
 import weakref
 
-from .livewindow import LiveWindow
 from .resource import Resource
-from .sensorbase import SensorBase
+from .sensorutil import SensorUtil
+from .sendablebuilder import SendableBuilder
 from .solenoidbase import SolenoidBase
 
 __all__ = ["DoubleSolenoid"]
 
-def _freeSolenoid(fwdHandle, revHandle):
+
+def _freeSolenoid(fwdHandle: hal.SolenoidHandle, revHandle: hal.SolenoidHandle) -> None:
     hal.freeSolenoidPort(fwdHandle)
     hal.freeSolenoidPort(revHandle)
 
+
 class DoubleSolenoid(SolenoidBase):
     """Controls 2 channels of high voltage Digital Output on the PCM.
 
@@ -31,11 +33,12 @@ class DoubleSolenoid(SolenoidBase):
 
     class Value(enum.IntEnum):
         """Possible values for a DoubleSolenoid."""
+
         kOff = 0
         kForward = 1
         kReverse = 2
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         """Constructor.
 
         Arguments can be supplied as positional or keyword.  Acceptable
@@ -56,8 +59,9 @@ class DoubleSolenoid(SolenoidBase):
         moduleNumber = kwargs.pop("moduleNumber", None)
 
         if kwargs:
-            warnings.warn("unknown keyword arguments: %s" % kwargs.keys(),
-                          RuntimeWarning)
+            warnings.warn(
+                "unknown keyword arguments: %s" % kwargs.keys(), RuntimeWarning
+            )
 
         # positional arguments
         if len(args) == 2:
@@ -65,25 +69,27 @@ class DoubleSolenoid(SolenoidBase):
         elif len(args) == 3:
             moduleNumber, forwardChannel, reverseChannel = args
         elif len(args) != 0:
-            raise ValueError("don't know how to handle %d positional arguments" % len(args))
+            raise ValueError(
+                "don't know how to handle %d positional arguments" % len(args)
+            )
 
         if moduleNumber is None:
-            moduleNumber = SensorBase.getDefaultSolenoidModule()
+            moduleNumber = SensorUtil.getDefaultSolenoidModule()
         if forwardChannel is None:
             raise ValueError("must specify forward channel")
         if reverseChannel is None:
             raise ValueError("must specify reverse channel")
 
         super().__init__(moduleNumber)
-        
+
         self.valueEntry = None
-        SensorBase.checkSolenoidModule(moduleNumber)
-        SensorBase.checkSolenoidChannel(forwardChannel)
-        SensorBase.checkSolenoidChannel(reverseChannel)
+        SensorUtil.checkSolenoidModule(moduleNumber)
+        SensorUtil.checkSolenoidChannel(forwardChannel)
+        SensorUtil.checkSolenoidChannel(reverseChannel)
 
         portHandle = hal.getPortWithModule(moduleNumber, forwardChannel)
         self.forwardHandle = hal.initializeSolenoidPort(portHandle)
-        
+
         try:
             portHandle = hal.getPortWithModule(moduleNumber, reverseChannel)
             self.reverseHandle = hal.initializeSolenoidPort(portHandle)
@@ -100,28 +106,30 @@ class DoubleSolenoid(SolenoidBase):
         # Need this to free on unit test wpilib reset
         Resource._add_global_resource(self)
 
-        hal.report(hal.UsageReporting.kResourceType_Solenoid,
-                      forwardChannel, moduleNumber)
-        hal.report(hal.UsageReporting.kResourceType_Solenoid,
-                      reverseChannel, moduleNumber)
+        hal.report(
+            hal.UsageReporting.kResourceType_Solenoid, forwardChannel, moduleNumber
+        )
+        hal.report(
+            hal.UsageReporting.kResourceType_Solenoid, reverseChannel, moduleNumber
+        )
 
         self.setName("DoubleSolenoid", moduleNumber, forwardChannel)
-        
-        self.__finalizer = weakref.finalize(self, _freeSolenoid,
-                                            self.forwardHandle, self.reverseHandle)
 
-    def free(self):
+        self.__finalizer = weakref.finalize(
+            self, _freeSolenoid, self.forwardHandle, self.reverseHandle
+        )
+
+    def close(self) -> None:
         """Mark the solenoid as freed."""
-        super().free()
+        super().close()
         self.__finalizer()
         self.forwardHandle = None
         self.reverseHandle = None
 
-    def set(self, value):
+    def set(self, value: Value) -> None:
         """Set the value of a solenoid.
 
         :param value: The value to set (Off, Forward, Reverse)
-        :type  value: :class:`DoubleSolenoid.Value`
         """
 
         if value == self.Value.kOff:
@@ -136,11 +144,10 @@ class DoubleSolenoid(SolenoidBase):
         else:
             raise ValueError("Invalid argument '%s'" % value)
 
-    def get(self):
+    def get(self) -> Value:
         """Read the current value of the solenoid.
 
         :returns: The current value of the solenoid.
-        :rtype: :class:`DoubleSolenoid.Value`
         """
         if hal.getSolenoid(self.forwardHandle):
             return self.Value.kForward
@@ -148,7 +155,7 @@ class DoubleSolenoid(SolenoidBase):
             return self.Value.kReverse
         return self.Value.kOff
 
-    def isFwdSolenoidBlackListed(self):
+    def isFwdSolenoidBlackListed(self) -> bool:
         """
         Check if the forward solenoid is blacklisted.
             If a solenoid is shorted, it is added to the blacklist and disabled until power cycle, or until faults are
@@ -160,7 +167,7 @@ class DoubleSolenoid(SolenoidBase):
 
         return (blacklist & self.forwardMask) != 0
 
-    def isRevSolenoidBlackListed(self):
+    def isRevSolenoidBlackListed(self) -> bool:
         """
         Check if the reverse solenoid is blacklisted.
             If a solenoid is shorted, it is added to the blacklist and disabled until power cycle, or until faults are
@@ -172,12 +179,15 @@ class DoubleSolenoid(SolenoidBase):
 
         return blacklist & (1 << self.reverseMask) != 0
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("Double Solenoid")
+        builder.setActuator(True)
         builder.setSafeState(lambda: self.set(self.Value.kOff))
-        builder.addStringProperty("Value", lambda: self.get().name[1:], self._valueChanged)
+        builder.addStringProperty(
+            "Value", lambda: self.get().name[1:], self._valueChanged
+        )
 
-    def _valueChanged(self, value):
+    def _valueChanged(self, value: str) -> None:
         if value == "Reverse":
             self.set(self.Value.kReverse)
         elif value == "Forward":
