@@ -1,4 +1,4 @@
-# validated: 2018-01-28 DV 48ae6c954a75 edu/wpi/first/wpilibj/DriverStation.java
+# validated: 2018-12-31 TW f0f196e5b389 edu/wpi/first/wpilibj/DriverStation.java
 # Copyright (c) 2008-2018 FIRST. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
@@ -7,6 +7,8 @@
 import threading
 
 import enum
+from typing import Optional
+
 import hal
 import sys
 import traceback
@@ -19,43 +21,50 @@ from .timer import Timer
 __all__ = ["DriverStation"]
 
 import logging
-logger = logging.getLogger('wpilib.ds')
+
+logger = logging.getLogger("wpilib.ds")
 
 JOYSTICK_UNPLUGGED_MESSAGE_INTERVAL = 1.0
 
 
 # NOTE: Not using hal.MatchInfo to avoid having to constantly decode the bytestrings.
 class MatchInfoData:
-    __slots__ = ('eventName', 'matchType', 'matchNumber', 'replayNumber', 'gameSpecificMessage')
+    __slots__ = (
+        "eventName",
+        "matchType",
+        "matchNumber",
+        "replayNumber",
+        "gameSpecificMessage",
+    )
 
-    def __init__(self):
-        self.eventName = ''
+    def __init__(self) -> None:
+        self.eventName = ""
         self.matchType = 0
         self.matchNumber = 0
         self.replayNumber = 0
-        self.gameSpecificMessage = ''
+        self.gameSpecificMessage = ""
 
 
 class MatchDataSender:
-    def __init__(self):
-        self.table = NetworkTables.getTable('FMSInfo')
-        self.typeMetadata = self.table.getEntry('.type')
-        self.typeMetadata.forceSetString('FMSInfo')
-        self.gameSpecificMessage = self.table.getEntry('GameSpecificMessage')
-        self.gameSpecificMessage.forceSetString('')
-        self.eventName = self.table.getEntry('EventName')
-        self.eventName.forceSetString('')
-        self.matchNumber = self.table.getEntry('MatchNumber')
+    def __init__(self) -> None:
+        self.table = NetworkTables.getTable("FMSInfo")
+        self.typeMetadata = self.table.getEntry(".type")
+        self.typeMetadata.forceSetString("FMSInfo")
+        self.gameSpecificMessage = self.table.getEntry("GameSpecificMessage")
+        self.gameSpecificMessage.forceSetString("")
+        self.eventName = self.table.getEntry("EventName")
+        self.eventName.forceSetString("")
+        self.matchNumber = self.table.getEntry("MatchNumber")
         self.matchNumber.forceSetDouble(0)
-        self.replayNumber = self.table.getEntry('ReplayNumber')
+        self.replayNumber = self.table.getEntry("ReplayNumber")
         self.replayNumber.forceSetDouble(0)
-        self.matchType = self.table.getEntry('MatchType')
+        self.matchType = self.table.getEntry("MatchType")
         self.matchType.forceSetDouble(0)
-        self.alliance = self.table.getEntry('IsRedAlliance')
+        self.alliance = self.table.getEntry("IsRedAlliance")
         self.alliance.forceSetBoolean(True)
-        self.station = self.table.getEntry('StationNumber')
+        self.station = self.table.getEntry("StationNumber")
         self.station.forceSetDouble(0)
-        self.controlWord = self.table.getEntry('FMSControlData')
+        self.controlWord = self.table.getEntry("FMSControlData")
         self.controlWord.forceSetDouble(0)
 
 
@@ -76,6 +85,7 @@ class DriverStation:
 
     class Alliance(enum.IntEnum):
         """The robot alliance that the robot is a part of."""
+
         Red = 0
         Blue = 1
         Invalid = 2
@@ -87,16 +97,16 @@ class DriverStation:
         Elimination = 3
 
     @classmethod
-    def _reset(cls):
-        ds = getattr(cls, 'instance', None)
+    def _reset(cls) -> None:
+        ds = getattr(cls, "instance", None)
         if ds is not None:
             ds.release()
-            #hal.giveMultiWait(ds.packetDataAvailableSem)
+            # hal.giveMultiWait(ds.packetDataAvailableSem)
             ds.thread.join()
             del cls.instance
 
     @classmethod
-    def getInstance(cls):
+    def getInstance(cls) -> "DriverStation":
         """Gets the global instance of the DriverStation.
 
         :returns: :class:`DriverStation`
@@ -108,16 +118,20 @@ class DriverStation:
             cls.instance = cls()
             return cls.instance
 
-    def __init__(self):
+    def __init__(self) -> None:
         """DriverStation constructor.
 
         The single DriverStation instance is created statically with the
         instance static member variable, you should never create a
         DriverStation instance.
         """
-        if not hasattr(DriverStation, 'instance') or DriverStation.instance is not None:
-            raise ValueError("Do not create DriverStation instances, use DriverStation.getInstance() instead")
+        if not hasattr(DriverStation, "instance") or DriverStation.instance is not None:
+            raise ValueError(
+                "Do not create DriverStation instances, use DriverStation.getInstance() instead"
+            )
 
+        # robotpy doesn't need to do this
+        # hal.initialize(500, 0)
         self.waitForDataCount = 0
         self.waitForDataCond = threading.Condition()
 
@@ -125,16 +139,24 @@ class DriverStation:
 
         self.joystickAxes = [hal.JoystickAxes() for _ in range(self.kJoystickPorts)]
         self.joystickPOVs = [hal.JoystickPOVs() for _ in range(self.kJoystickPorts)]
-        self.joystickButtons = [hal.JoystickButtons() for _ in range(self.kJoystickPorts)]
+        self.joystickButtons = [
+            hal.JoystickButtons() for _ in range(self.kJoystickPorts)
+        ]
         self.matchInfo = MatchInfoData()
 
-        self.joystickButtonsPressed = [hal.JoystickButtons() for _ in range(self.kJoystickPorts)]
-        self.joystickButtonsReleased = [hal.JoystickButtons() for _ in range(self.kJoystickPorts)]
+        self.joystickButtonsPressed = [0 for _ in range(self.kJoystickPorts)]
+        self.joystickButtonsReleased = [0 for _ in range(self.kJoystickPorts)]
 
-        self.joystickAxesCache = [hal.JoystickAxes() for _ in range(self.kJoystickPorts)]
-        self.joystickPOVsCache = [hal.JoystickPOVs() for _ in range(self.kJoystickPorts)]
-        self.joystickButtonsCache = [hal.JoystickButtons() for _ in range(self.kJoystickPorts)]
-        self.matchInfoCache = MatchInfoData()
+        self.joystickAxesCache = [
+            hal.JoystickAxes() for _ in range(self.kJoystickPorts)
+        ]
+        self.joystickPOVsCache = [
+            hal.JoystickPOVs() for _ in range(self.kJoystickPorts)
+        ]
+        self.joystickButtonsCache = [
+            hal.JoystickButtons() for _ in range(self.kJoystickPorts)
+        ]
+        self.matchInfoCache = hal.MatchInfo()
 
         self.controlWordMutex = threading.RLock()
         self.controlWordCache = hal.ControlWord()
@@ -159,12 +181,12 @@ class DriverStation:
         self.thread.daemon = True
         self.thread.start()
 
-    def release(self):
+    def release(self) -> None:
         """Kill the thread."""
         self.threadKeepAlive = False
 
     @staticmethod
-    def reportError(error, printTrace):
+    def reportError(error: str, printTrace: bool) -> None:
         """Report error to Driver Station, and also prints error to `sys.stderr`.
         Optionally appends stack trace to error message.
 
@@ -173,7 +195,7 @@ class DriverStation:
         DriverStation._reportErrorImpl(True, 1, error, printTrace)
 
     @staticmethod
-    def reportWarning(error, printTrace):
+    def reportWarning(error: str, printTrace: bool) -> None:
         """Report warning to Driver Station, and also prints error to `sys.stderr`.
         Optionally appends stack trace to error message.
 
@@ -182,7 +204,9 @@ class DriverStation:
         DriverStation._reportErrorImpl(False, 1, error, printTrace)
 
     @staticmethod
-    def _reportErrorImpl(isError, code, error, printTrace, exc_info=None):
+    def _reportErrorImpl(
+        isError: bool, code: int, error: str, printTrace: bool, exc_info=None
+    ) -> None:
         traceString = ""
         locString = ""
 
@@ -190,35 +214,143 @@ class DriverStation:
             # If an exception is passed in or an exception is present
             if exc_info is None:
                 exc_info = sys.exc_info()
-            
+
             exc = exc_info[0]
             if exc is None:
                 tb = traceback.extract_stack()[:-2]
             else:
                 tb = traceback.extract_tb(exc_info[2])
-            
+
             locString = "%s.%s:%s" % (tb[-1][0], tb[-1][1], tb[-1][2])
-            
-            trc = 'Traceback (most recent call last):\n'
-            stackstr = trc + ''.join(traceback.format_list(tb))
+
+            trc = "Traceback (most recent call last):\n"
+            stackstr = trc + "".join(traceback.format_list(tb))
             if exc is not None:
-                 stackstr += '  ' + (''.join(traceback.format_exception(*exc_info))).lstrip(trc)
-            traceString += '\n' + stackstr
-            
+                stackstr += "  " + (
+                    "".join(traceback.format_exception(*exc_info))
+                ).lstrip(trc)
+            traceString += "\n" + stackstr
+
             if exc is None:
                 logger.error(error + "\n" + traceString)
             else:
                 logger.error(error, exc_info=exc_info)
-            
+
         elif isError:
             logger.error(error)
         else:
             logger.warning(error)
 
-        hal.sendError(isError, code, False,
-                      error.encode('utf-8'),
-                      locString.encode('utf-8'),
-                      traceString.encode('utf-8'), True)
+        hal.sendError(
+            isError,
+            code,
+            False,
+            error.encode("utf-8"),
+            locString.encode("utf-8"),
+            traceString.encode("utf-8"),
+            True,
+        )
+
+    def getStickButton(self, stick: int, button: int) -> bool:
+        """The state of a button on the joystick. Button indexes begin at 1.
+
+        :param stick: The joystick port number
+        :param button: The button index, beginning at 1.
+
+        :returns: The state of the button.
+        """
+        if stick < 0 or stick >= self.kJoystickPorts:
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
+
+        if button <= 0:
+            self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
+            return False
+
+        with self.cacheDataMutex:
+            joystickButtons = self.joystickButtons[stick]
+            button_unavailable = button > joystickButtons.count
+            if not button_unavailable:
+                return ((0x1 << (button - 1)) & joystickButtons.buttons) != 0
+
+        self._reportJoystickUnpluggedWarning(
+            "Joystick Button %d on port %d not available, check if controller is plugged in"
+            % (button, stick)
+        )
+        return False
+
+    def getStickButtonPressed(self, stick: int, button: int) -> bool:
+        """Whether one joystick button was pressed since the last check.
+        Button indices begin at 1.
+
+        :param stick: Joystick to read
+        :param button: Button index, beginning at 1
+        :returns: Whether the joystick button was pressed since the last check
+        """
+        if button <= 0:
+            self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
+            return False
+        if not 0 <= stick < self.kJoystickPorts:
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
+
+        error = False
+        retVal = False
+        with self.cacheDataMutex:
+            if button > self.joystickButtons[stick].count:
+                error = True
+                retVal = False
+            else:
+                if self.joystickButtonsPressed[stick] & 1 << (button - 1):
+                    self.joystickButtonsPressed[stick] &= ~(1 << (button - 1))
+                    retVal = True
+                else:
+                    retVal = False
+        if error:
+            self._reportJoystickUnpluggedWarning(
+                "Joystick Button %s on port %s not available, check if controller is plugged in\n"
+                % (button, stick)
+            )
+
+        return retVal
+
+    def getStickButtonReleased(self, stick: int, button: int) -> bool:
+        """Whether one joystick button was released since the last check.
+        Button indices begin at 1.
+
+        :param stick: Joystick to read
+        :param button: Button index, beginning at 1
+        :returns: Whether the joystick button was released since the last check
+        """
+        if button <= 0:
+            self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
+            return False
+        if not 0 <= stick < self.kJoystickPorts:
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
+
+        error = False
+        retVal = False
+        with self.cacheDataMutex:
+            if button > self.joystickButtons[stick].count:
+                error = True
+                retVal = False
+            else:
+                if self.joystickButtonsReleased[stick] & 1 << (button - 1):
+                    self.joystickButtonsReleased[stick] &= ~(1 << (button - 1))
+                    retVal = True
+                else:
+                    retVal = False
+        if error:
+            self._reportJoystickUnpluggedWarning(
+                "Joystick Button %s on port %s not available, check if controller is plugged in\n"
+                % (button, stick)
+            )
+
+        return retVal
 
     def getStickAxis(self, stick: int, axis: int) -> float:
         """Get the value of the axis on a joystick.
@@ -234,13 +366,18 @@ class DriverStation:
         if axis < 0 or axis >= hal.kMaxJoystickAxes:
             raise IndexError("Joystick axis is out of range")
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             joystickAxes = self.joystickAxes[stick]
 
             if axis >= joystickAxes.count:
-                self._reportJoystickUnpluggedWarning("Joystick axis %d on port %d not available, check if controller is plugged in\n" % (axis, stick))
+                self._reportJoystickUnpluggedWarning(
+                    "Joystick axis %d on port %d not available, check if controller is plugged in\n"
+                    % (axis, stick)
+                )
                 return 0.0
 
             return joystickAxes.axes[axis]
@@ -257,15 +394,20 @@ class DriverStation:
         if pov < 0 or pov >= hal.kMaxJoystickPOVs:
             raise IndexError("Joystick POV is out of range")
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             joystickPOVs = self.joystickPOVs[stick]
+            pov_not_available = pov >= joystickPOVs.count
 
-            if pov >= joystickPOVs.count:
-                self._reportJoystickUnpluggedWarning("Joystick POV %d on port %d not available, check if controller is plugged in\n" % (pov, stick))
-                return -1
-            return joystickPOVs.povs[pov]
+        if pov_not_available:
+            self._reportJoystickUnpluggedWarning(
+                "Joystick POV %d on port %d not available, check if controller is plugged in\n"
+                % (pov, stick)
+            )
+        return joystickPOVs.povs[pov]
 
     def getStickButtons(self, stick: int) -> int:
         """The state of all the buttons on the joystick.
@@ -275,70 +417,13 @@ class DriverStation:
         :returns: The state of all buttons, as a bit array.
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             return self.joystickButtons[stick].buttons
 
-    def getStickButton(self, stick: int, button: int) -> bool:
-        """The state of a button on the joystick. Button indexes begin at 1.
-
-        :param stick: The joystick port number
-        :param button: The button index, beginning at 1.
-
-        :returns: The state of the button.
-        """
-        if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        with self.cacheDataMutex:
-            joystickButtons = self.joystickButtons[stick]
-            if button > joystickButtons.count:
-                self._reportJoystickUnpluggedWarning("Joystick Button %d on port %d not available, check if controller is plugged in" % (button, stick))
-                return False
-            if button <= 0:
-                self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
-                return False
-            return ((0x1 << (button - 1)) & joystickButtons.buttons) != 0
-
-    def getStickButtonPressed(self, stick: int, button: int) -> bool:
-        """Whether one joystick button was pressed since the last check.
-        Button indices begin at 1.
-
-        :param stick: Joystick to read
-        :param button: Button index, beginning at 1
-        :returns: Whether the joystick button was pressed since the last check
-        """
-        if button <= 0:
-            self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
-            return False
-        if not 0 <= stick < self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        if self.joystickButtonsPressed[stick].buttons & 1 << (button - 1):
-            self.joystickButtonsPressed[stick].buttons &= ~(1 << (button - 1))
-            return True
-        return False
-
-    def getStickButtonReleased(self, stick: int, button: int) -> bool:
-        """Whether one joystick button was released since the last check.
-        Button indices begin at 1.
-
-        :param stick: Joystick to read
-        :param button: Button index, beginning at 1
-        :returns: Whether the joystick button was released since the last check
-        """
-        if button <= 0:
-            self._reportJoystickUnpluggedError("Button indexes begin at 1 for WPILib")
-            return False
-        if not 0 <= stick < self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        if self.joystickButtonsReleased[stick].buttons & 1 << (button - 1):
-            self.joystickButtonsReleased[stick].buttons &= ~(1 << (button - 1))
-            return True
-        return False
-
     def getStickAxisCount(self, stick: int) -> int:
         """Returns the number of axes on a given joystick port.
 
@@ -347,7 +432,9 @@ class DriverStation:
         :returns: The number of axes on the indicated joystick
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             return self.joystickAxes[stick].count
@@ -360,7 +447,9 @@ class DriverStation:
         :returns: The number of POVs on the indicated joystick
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             return self.joystickPOVs[stick].count
@@ -373,7 +462,9 @@ class DriverStation:
         :returns: The number of buttons on the indicated joystick.
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         with self.cacheDataMutex:
             return self.joystickButtons[stick].count
@@ -386,14 +477,9 @@ class DriverStation:
         :returns: A boolean that returns the value of isXbox
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        with self.cacheDataMutex:
-            # TODO: Remove this when calling for descriptor on empty stick no longer crashes.
-            if 1 > self.joystickButtons[stick].count and 1 > len(self.joystickAxes[stick]):
-                self._reportJoystickUnpluggedWarning("WARNING: Joystick on port {} not avaliable, check if controller is "
-                                                     "plugged in.\n".format(stick))
-                return False
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         return hal.getJoystickIsXbox(stick)
 
@@ -406,14 +492,9 @@ class DriverStation:
         :returns: An integer that returns the value of type.
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        with self.cacheDataMutex:
-            # TODO: Remove this when calling for descriptor on empty stick no longer crashes.
-            if 1 > self.joystickButtons[stick].count and 1 > len(self.joystickAxes[stick]):
-                self._reportJoystickUnpluggedWarning("Joystick on port {} not avaliable, check if controller is "
-                                                     "plugged in.\n".format(stick))
-                return -1
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         return hal.getJoystickType(stick)
 
@@ -426,14 +507,9 @@ class DriverStation:
         :returns: The joystick name.
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
-
-        with self.cacheDataMutex:
-            # TODO: Remove this when calling for descriptor on empty stick no longer crashes.
-            if 1 > self.joystickButtons[stick].count and 1 > len(self.joystickAxes[stick].axes):
-                self._reportJoystickUnpluggedError("WARNING: Joystick on port {} not avaliable, check if controller is "
-                                                   "plugged in.\n".format(stick))
-                return ""
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
         return hal.getJoystickName(stick)
 
@@ -446,10 +522,11 @@ class DriverStation:
         :returns: An integer that reports type of axis the axis is reporting to be
         """
         if stick < 0 or stick >= self.kJoystickPorts:
-            raise IndexError("Joystick index is out of range, should be 0-%s" % self.kJoystickPorts)
+            raise IndexError(
+                "Joystick index is out of range, should be 0-%s" % self.kJoystickPorts
+            )
 
-        with self.cacheDataMutex:
-            return hal.getJoystickAxisType(stick, axis)
+        return hal.getJoystickAxisType(stick, axis)
 
     def isEnabled(self) -> bool:
         """Gets a value indicating whether the Driver Station requires the
@@ -459,7 +536,10 @@ class DriverStation:
         """
         with self.controlWordMutex:
             self._updateControlWord(False)
-            return self.controlWordCache.enabled != 0 and self.controlWordCache.dsAttached != 0
+            return (
+                self.controlWordCache.enabled != 0
+                and self.controlWordCache.dsAttached != 0
+            )
 
     def isDisabled(self) -> bool:
         """Gets a value indicating whether the Driver Station requires the
@@ -488,7 +568,9 @@ class DriverStation:
         """
         with self.controlWordMutex:
             self._updateControlWord(False)
-            return not (self.controlWordCache.autonomous != 0 or self.controlWordCache.test != 0)
+            return not (
+                self.controlWordCache.autonomous != 0 or self.controlWordCache.test != 0
+            )
 
     def isTest(self) -> bool:
         """Gets a value indicating whether the Driver Station requires the
@@ -528,7 +610,7 @@ class DriverStation:
             self._updateControlWord(False)
             return self.controlWordCache.fmsAttached != 0
 
-    def isSysActive(self):
+    def isSysActive(self) -> bool:
         """
         Gets a value indicating whether the FPGA outputs are enabled. The
         outputs may be disabled if the robot is disabled or e-stopped, the
@@ -541,7 +623,7 @@ class DriverStation:
         """
         return hal.getSystemActive()
 
-    def isBrownedOut(self):
+    def isBrownedOut(self) -> bool:
         """
         Check if the system is browned out.
 
@@ -618,7 +700,7 @@ class DriverStation:
         allianceStationID = hal.getAllianceStation()
         return self._station_numbers.get(allianceStationID, 0)
 
-    def waitForData(self, timeout: float = None) -> bool:
+    def waitForData(self, timeout: Optional[float] = None) -> bool:
         """Wait for new data or for timeout, which ever comes first.
 
         If timeout is None, wait for new data only.
@@ -629,7 +711,9 @@ class DriverStation:
         """
         with self.waitForDataCond:
             currentCount = self.waitForDataCount
-            signaled = self.waitForDataCond.wait_for(lambda: self.waitForDataCount != currentCount, timeout)
+            signaled = self.waitForDataCond.wait_for(
+                lambda: self.waitForDataCount != currentCount, timeout
+            )
             if not signaled:
                 # Return False if a timeout happened
                 return False
@@ -653,7 +737,7 @@ class DriverStation:
         """
         return hal.getMatchTime()
 
-    def getBatteryVoltage(self):
+    def getBatteryVoltage(self) -> float:
         """Read the battery voltage.
 
         .. deprecated:: 2018.0.0
@@ -663,7 +747,7 @@ class DriverStation:
         """
         return hal.getVinVoltage()
 
-    def InDisabled(self, entering):
+    def InDisabled(self, entering: bool) -> None:
         """Only to be used to tell the Driver Station what code you claim to
         be executing for diagnostic purposes only.
 
@@ -672,7 +756,7 @@ class DriverStation:
         """
         self.userInDisabled = entering
 
-    def InAutonomous(self, entering):
+    def InAutonomous(self, entering: bool) -> None:
         """Only to be used to tell the Driver Station what code you claim to
         be executing for diagnostic purposes only.
 
@@ -681,7 +765,7 @@ class DriverStation:
         """
         self.userInAutonomous = entering
 
-    def InOperatorControl(self, entering):
+    def InOperatorControl(self, entering: bool) -> None:
         """Only to be used to tell the Driver Station what code you claim to
         be executing for diagnostic purposes only.
 
@@ -690,7 +774,7 @@ class DriverStation:
         """
         self.userInTeleop = entering
 
-    def InTest(self, entering):
+    def InTest(self, entering: bool) -> None:
         """Only to be used to tell the Driver Station what code you claim to
         be executing for diagnostic purposes only.
 
@@ -699,7 +783,7 @@ class DriverStation:
         """
         self.userInTest = entering
 
-    def _sendMatchData(self):
+    def _sendMatchData(self) -> None:
         alliance = hal.getAllianceStation()
         hAid = hal.AllianceStationID
         isRedAlliance = alliance in {hAid.kRed1, hAid.kRed2, hAid.kRed3}
@@ -721,9 +805,9 @@ class DriverStation:
         self.matchDataSender.matchType.setDouble(matchType)
 
         with self.controlWordMutex:
-            self.matchDataSender.controlWord.setDouble(self.controlWordCache.to_bits())
+            self.matchDataSender.controlWord.setDouble(self.controlWordCache.bits)
 
-    def _getData(self):
+    def _getData(self) -> None:
         """Copy data from the DS task for the user.
         If no new data exists, it will just be returned, otherwise
         the data will be copied from the DS polling loop.
@@ -743,18 +827,37 @@ class DriverStation:
         # lock joystick mutex to swap cache data
         with self.cacheDataMutex:
             for i in range(self.kJoystickPorts):
-                self.joystickButtonsPressed[i].buttons |= \
-                    ~self.joystickButtons[i].buttons & self.joystickButtonsCache[i].buttons
+                self.joystickButtonsPressed[i] |= (
+                    ~self.joystickButtons[i].buttons
+                    & self.joystickButtonsCache[i].buttons
+                )
 
-                self.joystickButtonsReleased[i].buttons |= \
-                    self.joystickButtons[i].buttons & ~self.joystickButtonsCache[i].buttons
+                self.joystickButtonsReleased[i] |= (
+                    self.joystickButtons[i].buttons
+                    & ~self.joystickButtonsCache[i].buttons
+                )
 
             # move cache to actual data
-            self.joystickAxes, self.joystickAxesCache = self.joystickAxesCache, self.joystickAxes
-            self.joystickButtons, self.joystickButtonsCache = self.joystickButtonsCache, self.joystickButtons
-            self.joystickPOVs, self.joystickPOVsCache = self.joystickPOVsCache, self.joystickPOVs
+            self.joystickAxes, self.joystickAxesCache = (
+                self.joystickAxesCache,
+                self.joystickAxes,
+            )
+            self.joystickButtons, self.joystickButtonsCache = (
+                self.joystickButtonsCache,
+                self.joystickButtons,
+            )
+            self.joystickPOVs, self.joystickPOVsCache = (
+                self.joystickPOVsCache,
+                self.joystickPOVs,
+            )
 
-            self.matchInfo, self.matchInfoCache = self.matchInfoCache, self.matchInfo
+            self.matchInfo.eventName = self.matchInfoCache.eventName.decode("utf-8")
+            self.matchInfo.gameSpecificMessage = self.matchInfoCache.gameSpecificMessage.decode(
+                "utf-8"
+            )
+            self.matchInfo.matchNumber = self.matchInfoCache.matchNumber
+            self.matchInfo.replayNumber = self.matchInfoCache.replayNumber
+            self.matchInfo.matchType = self.matchInfoCache.matchType
 
         with self.waitForDataCond:
             self.waitForDataCount += 1
@@ -762,7 +865,7 @@ class DriverStation:
 
         self._sendMatchData()
 
-    def _reportJoystickUnpluggedError(self, message):
+    def _reportJoystickUnpluggedError(self, message: str) -> None:
         """
         Reports errors related to unplugged joysticks and throttles them so that they don't overwhelm the DS.
         """
@@ -771,7 +874,7 @@ class DriverStation:
             self.reportError(message, False)
             self.nextMessageTime = currentTime + JOYSTICK_UNPLUGGED_MESSAGE_INTERVAL
 
-    def _reportJoystickUnpluggedWarning(self, message):
+    def _reportJoystickUnpluggedWarning(self, message: str) -> None:
         """
         Reports errors related to unplugged joysticks and throttles them so that they don't overwhelm the DS.
         """
@@ -780,7 +883,7 @@ class DriverStation:
             self.reportWarning(message, False)
             self.nextMessageTime = currentTime + JOYSTICK_UNPLUGGED_MESSAGE_INTERVAL
 
-    def _run(self):
+    def _run(self) -> None:
         """Provides the service routine for the DS polling thread."""
         safetyCounter = 0
 
@@ -808,7 +911,7 @@ class DriverStation:
             if self.userInTest:
                 hal.observeUserProgramTest()
 
-    def _updateControlWord(self, force):
+    def _updateControlWord(self, force: bool) -> None:
         """Updates the data in the control word cache.
 
         Updates if the force parameter is set, or if 50ms have passed since the last update.
