@@ -1,10 +1,12 @@
-# validated: 2017-02-19 DS 12f759860eea edu/wpi/first/wpilibj/I2C.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN 0e9172f9a708 edu/wpi/first/wpilibj/I2C.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+import enum
+from typing import List, Union
 
 import hal
 import warnings
@@ -12,9 +14,11 @@ import weakref
 
 __all__ = ["I2C"]
 
-def _freeI2C(port):
+
+def _freeI2C(port: "I2C.Port") -> None:
     hal.closeI2C(port)
 
+
 class I2C:
     """I2C bus interface class.
 
@@ -28,15 +32,15 @@ class I2C:
         # Write bytes 'text', and receive 4 bytes in data
         data = i2c.transaction(b'text', 4)
     """
-    class Port:
+
+    class Port(enum.IntEnum):
         kOnboard = 0
         kMXP = 1
 
-    def __init__(self, port, deviceAddress, simPort=None):
+    def __init__(self, port: Port, deviceAddress: int, simPort=None) -> None:
         """Constructor.
 
         :param port: The I2C port the device is connected to.
-        :type port: :class:`.I2C.Port`
         :param deviceAddress: The address of the device on the I2C bus.
         :param simPort: This must be an object that implements all of
                         the i2c* functions from hal_impl that you use.
@@ -44,41 +48,52 @@ class I2C:
         """
         if port not in [self.Port.kOnboard, self.Port.kMXP]:
             raise ValueError("Invalid value '%s' for I2C port" % port)
-        
+
         if hal.HALIsSimulation():
             if simPort is None:
                 # If you want more functionality, implement your own mock
                 from hal_impl.i2c_helpers import I2CSimBase
+
                 simPort = I2CSimBase()
-                
+
                 msg = "Using stub simulator for I2C port %s" % port
                 warnings.warn(msg)
-            
+
             # Just check for basic functionality
-            assert hasattr(simPort, 'initializeI2C')
-            assert hasattr(simPort, 'closeI2C')
-            
+            assert hasattr(simPort, "initializeI2C")
+            assert hasattr(simPort, "closeI2C")
+
             self._port = (simPort, port)
         else:
             self._port = port
-            
+
         self.deviceAddress = deviceAddress
 
         hal.initializeI2C(self._port)
         self.__finalizer = weakref.finalize(self, _freeI2C, self._port)
 
         hal.report(hal.UsageReporting.kResourceType_I2C, deviceAddress)
-    
+
     @property
     def port(self):
         if not self.__finalizer.alive:
             raise ValueError("Cannot use i2c port after free() has been called")
         return self._port
-    
-    def free(self):
+
+    def free(self) -> None:
+        """
+        .. deprecated:: 2019.0.0
+            Use close instead
+        """
+        warnings.warn("use close instead", DeprecationWarning, stacklevel=2)
+        self.close()
+
+    def close(self) -> None:
         self.__finalizer()
 
-    def transaction(self, dataToSend, receiveSize):
+    def transaction(
+        self, dataToSend: Union[bytes, List[int]], receiveSize: int
+    ) -> bytes:
         """Generic transaction.
 
         This is a lower-level interface to the I2C hardware giving you more
@@ -88,16 +103,14 @@ class I2C:
         result in an error.
 
         :param dataToSend: Buffer of data to send as part of the transaction.
-        :type dataToSend: iterable of bytes
         :param receiveSize: Number of bytes to read from the device.
-        :type receiveSize: int
         :returns: Data received from the device.
-        :rtype: iterable of bytes
         """
-        return hal.transactionI2C(self.port, self.deviceAddress,
-                                  dataToSend, receiveSize)
+        return hal.transactionI2C(
+            self.port, self.deviceAddress, dataToSend, receiveSize
+        )
 
-    def addressOnly(self):
+    def addressOnly(self) -> bool:
         """Attempt to address a device on the I2C bus.
 
         This allows you to figure out if there is a device on the I2C bus that
@@ -111,7 +124,7 @@ class I2C:
             return True
         return False
 
-    def write(self, registerAddress, data):
+    def write(self, registerAddress: int, data: int) -> bool:
         """Execute a write transaction with the device.
 
         Write a single byte to a register on a device and wait until the
@@ -128,14 +141,13 @@ class I2C:
             return True
         return False
 
-    def writeBulk(self, data):
+    def writeBulk(self, data: bytes) -> bool:
         """Execute a write transaction with the device.
 
         Write multiple bytes to a register on a device and wait until the
         transaction is complete.
 
         :param data: The data to write to the device.
-        :type data: iterable of bytes
         :returns: Transfer Aborted... False for success, True for aborted.
         
         Usage::
@@ -152,7 +164,7 @@ class I2C:
             return True
         return False
 
-    def read(self, registerAddress, count):
+    def read(self, registerAddress: int, count: int) -> bytearray:
         """Execute a read transaction with the device.
 
         Read bytes from a device. Most I2C devices will auto-increment
@@ -162,13 +174,12 @@ class I2C:
         :param registerAddress: The register to read first in the transaction.
         :param count: The number of bytes to read in the transaction.
         :returns: The data read from the device.
-        :rtype: iterable of bytes
         """
         if count < 1:
             raise ValueError("count must be at least 1, %s given" % count)
         return self.transaction([registerAddress], count)
 
-    def readOnly(self, count):
+    def readOnly(self, count: int) -> bytes:
         """Execute a read only transaction with the device.
 
         Read bytes from a device. This method does not write any data
@@ -176,13 +187,12 @@ class I2C:
 
         :param count: The number of bytes to read in the transaction.
         :returns: The data read from the device.
-        :rtype: iterable of bytes
         """
         if count < 1:
             raise ValueError("count must be at least 1, %s given" % count)
         return hal.readI2C(self.port, self.deviceAddress, count)
 
-    def verifySensor(self, registerAddress, expected):
+    def verifySensor(self, registerAddress: int, expected: bytes) -> bool:
         """Verify that a device's registers contain expected values.
 
         Most devices will have a set of registers that contain a known value
