@@ -4,13 +4,15 @@
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 import warnings
+from typing import Optional
 
 from .generichid import GenericHID
 
 __all__ = ["JoystickBase"]
 
+
 class JoystickBase(GenericHID):
     """
     JoystickBase Interface.
@@ -19,11 +21,15 @@ class JoystickBase(GenericHID):
         Inherit directly from GenericHID instead.
     """
 
-    def __init__(self, port):
-        warnings.warn("JoystickBase is deprecated. Inherit directly from GenericHID instead", DeprecationWarning, stacklevel=2)
+    def __init__(self, port: int) -> None:
+        warnings.warn(
+            "JoystickBase is deprecated. Inherit directly from GenericHID instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         super().__init__(port)
 
-    def getZ(self, hand=None):
+    def getZ(self, hand: Optional[GenericHID.Hand] = None) -> float:
         """Get the z position of the HID.
 
         :param hand: which hand, left or right
@@ -32,7 +38,7 @@ class JoystickBase(GenericHID):
 
         raise NotImplementedError
 
-    def getTwist(self):
+    def getTwist(self) -> float:
         """Get the twist value.
 
         :returns: the twist value
@@ -40,7 +46,7 @@ class JoystickBase(GenericHID):
 
         raise NotImplementedError
 
-    def getThrottle(self):
+    def getThrottle(self) -> float:
         """Get the throttle.
 
         :returns: the throttle value
