@@ -1,17 +1,20 @@
 # validated: 2017-12-21 DV de134a5c608d edu/wpi/first/wpilibj/NamedSendable.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) 2012-2017 FIRST. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import warnings
 
+from ..command.subsystem import Subsystem
 from ..sendable import Sendable
+from ..sendablebuilder import SendableBuilder
 
 __all__ = ["NamedSendable"]
 
+
 class NamedSendable(Sendable):
     """The interface for sendable objects that gives the sendable a default
     name in the Smart Dashboard.
@@ -20,10 +23,13 @@ class NamedSendable(Sendable):
         Use :class:`.Sendable` directly instead.
     """
 
-    def __init_subclass__(cls, **kwargs):
+    def __init_subclass__(cls, **kwargs) -> None:
         super().__init_subclass__(**kwargs)
-        warnings.warn('NamedSendable is deprecated, use Sendable directly instead',
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "NamedSendable is deprecated, use Sendable directly instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
 
     def getName(self) -> str:
         """The name of the subtable.
@@ -33,14 +39,14 @@ class NamedSendable(Sendable):
         """
         raise NotImplementedError
 
-    def setName(self, name):
+    def setName(self, name: str) -> None:
         pass
 
-    def getSubsystem(self):
+    def getSubsystem(self) -> str:
         return ""
 
-    def setSubsystem(self, subsystem):
+    def setSubsystem(self, subsystem: Subsystem) -> None:
         pass
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         pass
