@@ -1,21 +1,21 @@
-# validated: 2018-01-06 TW f9bece2ffbf7 edu/wpi/first/wpilibj/InterruptableSensorBase.java
+# validated: 2018-12-18 n 9207d788ab50 edu/wpi/first/wpilibj/InterruptableSensorBase.java
 # ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
 # ----------------------------------------------------------------------------
-
+from typing import Callable, Optional
 import enum
 import weakref
 
 import hal
-from .sensorbase import SensorBase
+from .sendablebase import SendableBase
 
 __all__ = ["InterruptableSensorBase"]
 
 
-class InterruptableSensorBase(SensorBase):
+class InterruptableSensorBase(SendableBase):
     """Base for sensors to be used with interrupts"""
 
     class WaitResult(enum.IntEnum):
@@ -24,7 +24,7 @@ class InterruptableSensorBase(SensorBase):
         kFallingEdge = 0x100
         kBoth = 0x101
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Create a new InterrupatableSensorBase"""
         super().__init__()
         # The interrupt resource
@@ -34,25 +34,27 @@ class InterruptableSensorBase(SensorBase):
         self.isSynchronousInterrupt = False
 
     @property
-    def interrupt(self):
+    def interrupt(self) -> Optional[hal.InterruptHandle]:
         if self._interrupt_finalizer is None:
             return None
         if not self._interrupt_finalizer.alive:
             return None
         return self._interrupt
 
-    def free(self):
-        super().free()
+    def close(self) -> None:
+        super().close()
         if self.interrupt is not None:
             self.cancelInterrupts()
 
-    def getAnalogTriggerTypeForRouting(self):
+    def getAnalogTriggerTypeForRouting(self) -> int:
         raise NotImplementedError
 
-    def getPortHandleForRouting(self):
+    def getPortHandleForRouting(self) -> int:
         raise NotImplementedError
 
-    def requestInterrupts(self, handler=None):
+    def requestInterrupts(
+        self, handler: Optional[Callable[[int], None]] = None
+    ) -> None:
         """Request one of the 8 interrupts asynchronously on this digital
         input.
 
@@ -71,11 +73,14 @@ class InterruptableSensorBase(SensorBase):
 
         assert self.interrupt is not None
 
-        hal.requestInterrupts(self.interrupt, self.getPortHandleForRouting(),
-                              self.getAnalogTriggerTypeForRouting())
+        hal.requestInterrupts(
+            self.interrupt,
+            self.getPortHandleForRouting(),
+            self.getAnalogTriggerTypeForRouting(),
+        )
         self.setUpSourceEdge(True, False)
 
-    def allocateInterrupts(self, watcher):
+    def allocateInterrupts(self, watcher: bool) -> None:
         """Allocate the interrupt
 
         :param watcher: True if the interrupt should be in synchronous mode
@@ -86,9 +91,11 @@ class InterruptableSensorBase(SensorBase):
             raise ValueError("The interrupt has already been allocated")
         self.isSynchronousInterrupt = watcher
         self._interrupt = hal.initializeInterrupts(watcher)
-        self._interrupt_finalizer = weakref.finalize(self, hal.cleanInterrupts, self._interrupt)
+        self._interrupt_finalizer = weakref.finalize(
+            self, hal.cleanInterrupts, self._interrupt
+        )
 
-    def cancelInterrupts(self):
+    def cancelInterrupts(self) -> None:
         """Cancel interrupts on this device. This deallocates all the
         chipobject structures and disables any interrupts.
         """
@@ -98,7 +105,7 @@ class InterruptableSensorBase(SensorBase):
         hal.cleanInterrupts(self.interrupt)
         self.interrupt = None
 
-    def waitForInterrupt(self, timeout, ignorePrevious=True):
+    def waitForInterrupt(self, timeout: float, ignorePrevious: bool = True) -> int:
         """In synchronous mode, wait for the defined interrupt to occur.
         You should **NOT** attempt to read the sensor from another thread
         while waiting for an interrupt. This is not threadsafe, and can cause 
@@ -117,7 +124,7 @@ class InterruptableSensorBase(SensorBase):
         result = rising | falling
         return self.WaitResult(result)
 
-    def enableInterrupts(self):
+    def enableInterrupts(self) -> None:
         """Enable interrupts to occur on this input. Interrupts are disabled
         when the RequestInterrupt call is made. This gives time to do the
         setup of the other options before starting to field interrupts.
@@ -128,7 +135,7 @@ class InterruptableSensorBase(SensorBase):
             raise ValueError("You do not need to enable synchronous interrupts")
         hal.enableInterrupts(self.interrupt)
 
-    def disableInterrupts(self):
+    def disableInterrupts(self) -> None:
         """Disable Interrupts without without deallocating structures."""
         if self.interrupt is None:
             raise ValueError("The interrupt is not allocated.")
@@ -136,7 +143,7 @@ class InterruptableSensorBase(SensorBase):
             raise ValueError("You can not disable synchronous interrupts")
         hal.disableInterrupts(self.interrupt)
 
-    def readRisingTimestamp(self):
+    def readRisingTimestamp(self) -> float:
         """Return the timestamp for the rising interrupt that occurred most
         recently.  This is in the same time domain as getClock().  The
         rising-edge interrupt should be enabled with setUpSourceEdge.
@@ -145,9 +152,9 @@ class InterruptableSensorBase(SensorBase):
         """
         if self.interrupt is None:
             raise ValueError("The interrupt is not allocated.")
-        return hal.readInterruptRisingTimestamp(self.interrupt)
+        return hal.readInterruptRisingTimestamp(self.interrupt) * 1e-6
 
-    def readFallingTimestamp(self):
+    def readFallingTimestamp(self) -> float:
         """Return the timestamp for the falling interrupt that occurred most
         recently.  This is in the same time domain as getClock().  The
         falling-edge interrupt should be enabled with setUpSourceEdge.
@@ -156,17 +163,17 @@ class InterruptableSensorBase(SensorBase):
         """
         if self.interrupt is None:
             raise ValueError("The interrupt is not allocated.")
-        return hal.readInterruptFallingTimestamp(self.interrupt)
+        return hal.readInterruptFallingTimestamp(self.interrupt) * 1e-6
 
-    def setUpSourceEdge(self, risingEdge, fallingEdge):
+    def setUpSourceEdge(self, risingEdge: bool, fallingEdge: bool) -> None:
         """Set which edge to trigger interrupts on
 
         :param risingEdge: True to interrupt on rising edge
         :param fallingEdge: True to interrupt on falling edge
         """
         if self.interrupt is not None:
-            hal.setInterruptUpSourceEdge(self.interrupt,
-                                         1 if risingEdge else 0,
-                                         1 if fallingEdge else 0)
+            hal.setInterruptUpSourceEdge(
+                self.interrupt, 1 if risingEdge else 0, 1 if fallingEdge else 0
+            )
         else:
             raise ValueError("You must call RequestInterrupts before setUpSourceEdge")
