@@ -1,10 +1,10 @@
 # validated: 2017-12-15 EN f9bece2ffbf7 edu/wpi/first/wpilibj/Jaguar.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
@@ -12,12 +12,13 @@ from .pwmspeedcontroller import PWMSpeedController
 
 __all__ = ["Jaguar"]
 
+
 class Jaguar(PWMSpeedController):
     """
         Texas Instruments / Vex Robotics Jaguar Speed Controller as a PWM device.
     """
 
-    def __init__(self, channel):
+    def __init__(self, channel: int) -> None:
         """Constructor.
 
         :param channel: The PWM channel that the Jaguar is attached to. 0-9 are on-board, 10-19 are on the MXP port
@@ -34,7 +35,6 @@ class Jaguar(PWMSpeedController):
         self.setPeriodMultiplier(self.PeriodMultiplier.k1X)
         self.setSpeed(0)
         self.setZeroLatch()
-        
-        hal.report(hal.UsageReporting.kResourceType_Jaguar,
-                   self.getChannel())
+
+        hal.report(hal.UsageReporting.kResourceType_Jaguar, self.getChannel())
         self.setName("Jaguar", self.getChannel())
