@@ -1,4 +1,4 @@
-# validated: 2017-11-13 TW 595b1df380f7 edu/wpi/first/wpilibj/Joystick.java
+# validated: 2018-09-30 EN 64b03704f8db edu/wpi/first/wpilibj/Joystick.java
 # ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -25,6 +25,12 @@ class Joystick(GenericHID):
     hardware buttons depends on the code in the Driver Station.
     """
 
+    kDefaultXChannel = 0
+    kDefaultYChannel = 1
+    kDefaultZChannel = 2
+    kDefaultTwistChannel = 2
+    kDefaultThrottleChannel = 3
+
     kDefaultXAxis = 0
     kDefaultYAxis = 1
     kDefaultZAxis = 2
@@ -35,6 +41,7 @@ class Joystick(GenericHID):
 
     class AxisType(enum.IntEnum):
         """Represents an analog axis on a joystick."""
+
         kX = 0
         kY = 1
         kZ = 2
@@ -45,6 +52,7 @@ class Joystick(GenericHID):
     # This copy is here because of the extra kNumAxes.
     class Axis(enum.IntEnum):
         """Represents an analog axis on a joystick"""
+
         kX = 0
         kY = 1
         kZ = 2
@@ -54,6 +62,7 @@ class Joystick(GenericHID):
 
     class ButtonType(enum.IntEnum):
         """Represents a digital button on the Joystick"""
+
         kTrigger = 1
         kTop = 2
 
@@ -74,62 +83,63 @@ class Joystick(GenericHID):
         super().__init__(port)
 
         self.axes = [0] * self.Axis.kNumAxes
-        self.axes[self.Axis.kX] = self.kDefaultXAxis
-        self.axes[self.Axis.kY] = self.kDefaultYAxis
-        self.axes[self.Axis.kZ] = self.kDefaultZAxis
-        self.axes[self.Axis.kTwist] = self.kDefaultTwistAxis
-        self.axes[self.Axis.kThrottle] = self.kDefaultThrottleAxis
+        self.axes[self.Axis.kX] = self.kDefaultXChannel
+        self.axes[self.Axis.kY] = self.kDefaultYChannel
+        self.axes[self.Axis.kZ] = self.kDefaultZChannel
+        self.axes[self.Axis.kTwist] = self.kDefaultTwistChannel
+        self.axes[self.Axis.kThrottle] = self.kDefaultThrottleChannel
 
         hal.report(hal.UsageReporting.kResourceType_Joystick, port)
 
-    def setXChannel(self, channel: int):
+    def setXChannel(self, channel: int) -> None:
         """Set the channel associated with the X axis.
 
         :param channel: The channel to set the axis to.
         """
         self.axes[self.Axis.kX] = channel
 
-    def setYChannel(self, channel: int):
+    def setYChannel(self, channel: int) -> None:
         """Set the channel associated with the Y axis.
 
         :param channel: The channel to set the axis to.
         """
         self.axes[self.Axis.kY] = channel
 
-    def setZChannel(self, channel: int):
+    def setZChannel(self, channel: int) -> None:
         """Set the channel associated with the Z axis.
 
         :param channel: The channel to set the axis to.
         """
         self.axes[self.Axis.kZ] = channel
 
-    def setThrottleChannel(self, channel: int):
+    def setThrottleChannel(self, channel: int) -> None:
         """Set the channel associated with the Throttle axis.
 
         :param channel: The channel to set the axis to.
         """
         self.axes[self.Axis.kThrottle] = channel
 
-    def setTwistChannel(self, channel: int) -> int:
+    def setTwistChannel(self, channel: int) -> None:
         """Set the channel associated with the Twist axis.
 
         :param channel: The channel to set the axis to.
         """
         self.axes[self.Axis.kTwist] = channel
 
-    def setAxisChannel(self, axis, channel):
+    def setAxisChannel(self, axis, channel) -> None:
         """Set the channel associated with a specified axis.
 
         :param axis: The axis to set the channel for.
-        :type  axis: int
         :param channel: The channel to set the axis to.
-        :type  channel: int
 
         .. deprecated:: 2018.0.0
             Use the more specific axis channel setter functions
         """
-        warnings.warn("setAxisChannel is deprecated. Use the more specific axis channel setter functions",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "setAxisChannel is deprecated. Use the more specific axis channel setter functions",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         self.axes[axis] = channel
 
     def getXChannel(self) -> int:
@@ -176,8 +186,11 @@ class Joystick(GenericHID):
         ..deprecated:: 2018.0.0
             Use the more specific axis channel getter functions
         """
-        warnings.warn("getAxisChannel is deprecated. Use the more specific axis channel getter functions",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "getAxisChannel is deprecated. Use the more specific axis channel getter functions",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.axes[axis]
 
     def getX(self, hand=None) -> float:
@@ -239,14 +252,16 @@ class Joystick(GenericHID):
         example :func:`getX`).
 
         :param axis: The axis to read.
-        :type axis: :class:`Joystick.AxisType`
         :returns: The value of the axis.
 
         ..deprecated: 2018.0.0
             Use the more specific axis getter functions.
         """
-        warnings.warn("getAxis is deprecated. Use the more specific axis setter functions",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "getAxis is deprecated. Use the more specific axis setter functions",
+            DeprecationWarning,
+            stacklevel=2,
+        )
 
         if axis == self.AxisType.kX:
             return self.getX()
@@ -260,7 +275,8 @@ class Joystick(GenericHID):
             return self.getThrottle()
         else:
             raise ValueError(
-                "Invalid axis specified! Must be one of wpilib.Joystick.AxisType, or use getRawAxis instead")
+                "Invalid axis specified! Must be one of wpilib.Joystick.AxisType, or use getRawAxis instead"
+            )
 
     def getTrigger(self) -> bool:
         """Read the state of the trigger on the joystick.
@@ -272,14 +288,14 @@ class Joystick(GenericHID):
         """
         return self.getRawButton(self.Button.kTrigger)
 
-    def getTriggerPressed(self):
+    def getTriggerPressed(self) -> bool:
         """Whether the trigger was pressed since the last check
 
         :returns: Whether the button was pressed since the last check
         """
         return self.getRawButtonPressed(self.Button.kTrigger)
 
-    def getTriggerReleased(self):
+    def getTriggerReleased(self) -> bool:
         """Whether the trigger was released since the last check.
 
         :returns: Whether the button was released since the last check.
@@ -295,14 +311,14 @@ class Joystick(GenericHID):
         """
         return self.getRawButton(self.Button.kTop)
 
-    def getTopPressed(self):
+    def getTopPressed(self) -> bool:
         """Whether the trigger was pressed since the last check
 
         :returns: Whether the button was pressed since the last check
         """
         return self.getRawButtonPressed(self.Button.kTop)
 
-    def getTopReleased(self):
+    def getTopReleased(self) -> bool:
         """Whether the trigger was released since the last check.
 
         :returns: Whether the button was released since the last check.
@@ -320,15 +336,19 @@ class Joystick(GenericHID):
         ..deprecated: 2018.0.0
             Use Button enum values instead of ButtonType
         """
-        warnings.warn("getButton is deprecated. Use Button enum values instead of ButtonType",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "getButton is deprecated. Use Button enum values instead of ButtonType",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if button == self.ButtonType.kTrigger:
             return self.getTrigger()
         elif button == self.ButtonType.kTop:
             return self.getTop()
         else:
             raise ValueError(
-                "Invalid button specified! Must be one of wpilib.Joystick.ButtonType, or use getRawButton instead")
+                "Invalid button specified! Must be one of wpilib.Joystick.ButtonType, or use getRawButton instead"
+            )
 
     def getMagnitude(self) -> float:
         """Get the magnitude of the direction vector formed by the joystick's
