@@ -1,23 +1,29 @@
-# validated: 2017-12-09 EN f9bece2ffbf7 edu/wpi/first/wpilibj/livewindow/LiveWindow.java
+# validated: 2019-01-02 DV 97ba195b881e edu/wpi/first/wpilibj/livewindow/LiveWindow.java
 # ----------------------------------------------------------------------------
-#  Copyright (c) 2008-2017 FIRST. All Rights Reserved.                        
-#  Open Source Software - may be modified and shared by FRC teams. The code   
-#  must be accompanied by the FIRST BSD license file in the root directory of 
-#  the project.                                                               
+#  Copyright (c) 2008-2017 FIRST. All Rights Reserved.
+#  Open Source Software - may be modified and shared by FRC teams. The code
+#  must be accompanied by the FIRST BSD license file in the root directory of
+#  the project.
 # ----------------------------------------------------------------------------
+import logging
 import threading
+import warnings
+from typing import Optional, Union
+
 from networktables import NetworkTablesInstance
+from networktables.entry import NetworkTableEntry
+from networktables.networktable import NetworkTable
+from .sendable import Sendable
 from .sendablebuilder import SendableBuilder
 
-import warnings
-import logging
+
 logger = logging.getLogger(__name__)
 
 __all__ = ["LiveWindow"]
 
 
 class Component:
-    def __init__(self, sendable, parent):
+    def __init__(self, sendable: Sendable, parent: Optional[Sendable]) -> None:
         self.sendable = sendable
         self.parent = parent
         self.builder = SendableBuilder()
@@ -30,11 +36,13 @@ class _LiveWindowComponent:
     added to the SmartDashboard in test mode. The components are cached until
     the first time the robot enters Test mode. This allows the components to
     be inserted, then renamed."""
-    def __init__(self, subsystem, name, isSensor):
+
+    def __init__(self, subsystem, name: str, isSensor: bool) -> None:
         self.subsystem = subsystem
         self.name = str(name)
         self.isSensor = isSensor
 
+
 class LiveWindow:
     """The public interface for putting sensors and
     actuators on the LiveWindow."""
@@ -49,25 +57,27 @@ class LiveWindow:
     mutex = threading.RLock()
 
     @classmethod
-    def liveWindowTable(cls):
+    def liveWindowTable(cls) -> NetworkTable:
         if cls._liveWindowTable is None:
-            cls._liveWindowTable = NetworkTablesInstance.getDefault().getTable("LiveWindow")
+            cls._liveWindowTable = NetworkTablesInstance.getDefault().getTable(
+                "LiveWindow"
+            )
         return cls._liveWindowTable
 
     @classmethod
-    def statusTable(cls):
-        if cls._statusTable is None: 
+    def statusTable(cls) -> NetworkTable:
+        if cls._statusTable is None:
             cls._statusTable = cls.liveWindowTable().getSubTable(".status")
         return cls._statusTable
 
     @classmethod
-    def enabledEntry(cls):
+    def enabledEntry(cls) -> NetworkTableEntry:
         if cls._enabledEntry is None:
             cls._enabledEntry = cls.statusTable().getEntry("LW Enabled")
         return cls._enabledEntry
-    
+
     @classmethod
-    def _reset(cls):
+    def _reset(cls) -> None:
         cls.components = {}
         cls._liveWindowTable = None
         cls._statusTable = None
@@ -77,11 +87,11 @@ class LiveWindow:
         cls.telemetryEnabled = True
 
     @classmethod
-    def isEnabled(cls):
+    def isEnabled(cls) -> bool:
         return cls.liveWindowEnabled
 
     @classmethod
-    def setEnabled(cls, enabled):
+    def setEnabled(cls, enabled: bool) -> None:
         """Set the enabled state of LiveWindow. If it's being enabled, turn
         off the scheduler and remove all the commands from the queue and
         enable all the components registered for LiveWindow. If it's being
@@ -94,35 +104,44 @@ class LiveWindow:
         after a period of adjusting them in LiveWindow mode.
         """
         from .command import Scheduler
-        if cls.liveWindowEnabled != enabled:
-            scheduler = Scheduler.getInstance()
-            if enabled:
-                logger.info("Starting live window mode.")
-                scheduler.disable()
-                scheduler.removeAll()
-            else:
-                logger.info("Stopping live window mode.")
-                for component in cls.components.values():
-                    component.builder.stopLiveWindowMode()
-                scheduler.enable()
-            cls.startLiveWindow = enabled
-            cls.liveWindowEnabled = enabled
-            cls.enabledEntry().setBoolean(enabled)
+
+        with cls.mutex:
+            if cls.liveWindowEnabled != enabled:
+                cls.startLiveWindow = enabled
+                cls.liveWindowEnabled = enabled
+                cls.updateValues()  # Force table generation now to make sure everything is defined
+                scheduler = Scheduler.getInstance()
+                if enabled:
+                    logger.info("Starting live window mode.")
+                    scheduler.disable()
+                    scheduler.removeAll()
+                else:
+                    logger.info("Stopping live window mode.")
+                    components = list(cls.components.values())
+                    for component in components:
+                        component.builder.stopLiveWindowMode()
+                    scheduler.enable()
+                cls.enabledEntry().setBoolean(enabled)
 
     @classmethod
-    def run(cls):
+    def run(cls) -> None:
         """The run method is called repeatedly to keep the values refreshed
         on the screen in test mode.
 
         .. deprecated:: 2018.0.0
             No longer required
         """
-        warnings.warn("run is deprecated. It is no longer required.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "run is deprecated. It is no longer required.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         cls.updateValues()
 
     @classmethod
-    def addSensor(cls, subsystem, name, component):
+    def addSensor(
+        cls, subsystem: str, name: Union[str, int], component: Sendable
+    ) -> None:
         """Add a Sensor associated with the subsystem and with call it by the
         given name.
 
@@ -134,15 +153,18 @@ class LiveWindow:
         .. deprecated:: 2018.0.0
             Use :meth:`.Sendable.setName` instead.
         """
-        warnings.warn("addSensor is deprecated. " + 
-                      "Use Sendable.setName(subsystem, name) instead.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "addSensor is deprecated. "
+            + "Use Sendable.setName(subsystem, name) instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         with cls.mutex:
             cls.add(component)
             component.setName(subsystem, name)
 
     @classmethod
-    def addActuator(cls, subsystem, name, component):
+    def addActuator(cls, subsystem: str, name: str, component: Sendable) -> None:
         """Add an Actuator associated with the subsystem and with call it by
         the given name.
 
@@ -153,15 +175,20 @@ class LiveWindow:
         .. deprecated:: 2018.0.0
             Use :meth:`.Sendable.setName` instead.
         """
-        warnings.warn("addActuator is deprecated. " +
-                      "Use Sendable.setName(subsystem, name) instead.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "addActuator is deprecated. "
+            + "Use Sendable.setName(subsystem, name) instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         with cls.mutex:
             cls.add(component)
             component.setName(subsystem, name)
 
     @classmethod
-    def addSensorChannel(cls, moduleType, channel, component):
+    def addSensorChannel(
+        cls, moduleType: str, channel: int, component: Sendable
+    ) -> None:
         """Add Sensor to LiveWindow. The components are shown with the type
         and channel like this: Gyro[0] for a gyro object connected to the
         first analog channel.
@@ -174,14 +201,19 @@ class LiveWindow:
         .. deprecated:: 2018.0.0
             Use :meth:`.SendableBase.setName` instead.
         """
-        warnings.warn("addSensorChannel is deprecated. "+ 
-                      "Use SendableBase.setName(moduleType, channel) instead.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "addSensorChannel is deprecated. "
+            + "Use SendableBase.setName(moduleType, channel) instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         cls.add(component)
         component.setName("Ungrouped", "%s[%s]" % (moduleType, channel))
 
     @classmethod
-    def addActuatorChannel(cls, moduleType, channel, component):
+    def addActuatorChannel(
+        cls, moduleType: str, channel: int, component: Sendable
+    ) -> None:
         """Add Actuator to LiveWindow. The components are shown with the
         module type, slot and channel like this: Servo[0,2] for a servo
         object connected to the first digital module and PWM port 2.
@@ -195,14 +227,19 @@ class LiveWindow:
         .. deprecated:: 2018.0.0
             Use :meth:`.SendableBase.setName` instead.
         """
-        warnings.warn("addActuatorChannel is deprecated. " + 
-                      "Use SendableBase.setName(moduleType, channel) instead.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "addActuatorChannel is deprecated. "
+            + "Use SendableBase.setName(moduleType, channel) instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         cls.add(component)
         component.setName("Ungrouped", "%s[%s]" % (moduleType, channel))
 
     @classmethod
-    def addActuatorModuleChannel(cls, moduleType, moduleNumber, channel, component):
+    def addActuatorModuleChannel(
+        cls, moduleType: str, moduleNumber: int, channel: int, component: Sendable
+    ) -> None:
         """Add Actuator to LiveWindow. The components are shown with the
         module type, slot and channel like this: Servo[0,2] for a servo
         object connected to the first digital module and PWM port 2.
@@ -217,14 +254,19 @@ class LiveWindow:
         .. deprecated:: 2018.0.0
             Use :meth:`.SendableBase.setName` instead.
         """
-        warnings.warn("addActuatorModuleChannel is deprecated. " + 
-                      "Use SendableBase.setName(moduleType, moduleNumber, channel) instead.",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "addActuatorModuleChannel is deprecated. "
+            + "Use SendableBase.setName(moduleType, moduleNumber, channel) instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         cls.add(component)
-        component.setName("Ungrouped", "%s[%s,%s]" % (moduleType, moduleNumber, channel))
+        component.setName(
+            "Ungrouped", "%s[%s,%s]" % (moduleType, moduleNumber, channel)
+        )
 
     @classmethod
-    def add(cls, sendable):
+    def add(cls, sendable: Sendable) -> None:
         """
         Add a component to the LiveWindow.
 
@@ -235,7 +277,7 @@ class LiveWindow:
                 cls.components[sendable] = Component(sendable, None)
 
     @classmethod
-    def addChild(cls, parent, child):
+    def addChild(cls, parent: Sendable, child: object) -> None:
         """
         Add a child component to a component.
 
@@ -250,13 +292,13 @@ class LiveWindow:
             else:
                 component.parent = parent
             component.telemetryEnabled = False
-        
+
     @classmethod
-    def remove(cls, sendable):
+    def remove(cls, sendable: Sendable) -> None:
         """
         Remove a component from the LiveWindow.
 
-        @param sendable component to remove
+        :param sendable: component to remove
         """
         with cls.mutex:
             if sendable in cls.components:
@@ -266,7 +308,7 @@ class LiveWindow:
                     component.builder.stopLiveWindowMode()
 
     @classmethod
-    def enableTelemetry(cls, sendable):
+    def enableTelemetry(cls, sendable: Sendable) -> None:
         """
         Enable telemetry for a single component.
 
@@ -279,7 +321,7 @@ class LiveWindow:
                 component.telemetryEnabled = True
 
     @classmethod
-    def disableTelemetry(cls, sendable):
+    def disableTelemetry(cls, sendable: Sendable) -> None:
         """
         Disable telemetry for a single component.  
 
@@ -291,7 +333,7 @@ class LiveWindow:
                 component.telemetryEnabled = False
 
     @classmethod
-    def disableAllTelemetry(cls):
+    def disableAllTelemetry(cls) -> None:
         """ Disable ALL telemetry """
         with cls.mutex:
             cls.telemetryEnabled = False
@@ -299,14 +341,19 @@ class LiveWindow:
                 component.telemetryEnabled = False
 
     @classmethod
-    def updateValues(cls):
+    def updateValues(cls) -> None:
         with cls.mutex:
             # only do this if either LiveWindow mode or telemetry is enabled
             if not cls.liveWindowEnabled and not cls.telemetryEnabled:
                 return
 
-            for component in cls.components.values():
-                if component.sendable is not None and component.parent is None and (cls.liveWindowEnabled or component.telemetryEnabled):
+            components = list(cls.components.values())
+            for component in components:
+                if (
+                    component.sendable is not None
+                    and component.parent is None
+                    and (cls.liveWindowEnabled or component.telemetryEnabled)
+                ):
                     if component.firstTime:
                         # By holding off creating the NetworkTable entries, it allows the
                         # components to be redefined. This allows default sensor and actuator
@@ -333,4 +380,3 @@ class LiveWindow:
                     component.builder.updateTable()
 
             cls.startLiveWindow = False
-
