@@ -1,12 +1,13 @@
-# novalidate
-#----------------------------------------------------------------------------
+# validated: 2019-01-04 DS 3178911eef5 edu/wpi/first/wpilibj/Preferences.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import logging
+from typing import List, Optional
 
 from networktables import NetworkTables
 
@@ -41,7 +42,7 @@ class Preferences:
     TABLE_NAME = "Preferences"
 
     @staticmethod
-    def getInstance():
+    def getInstance() -> "Preferences":
         """Returns the preferences instance.
 
         :returns: the preferences instance
@@ -51,33 +52,36 @@ class Preferences:
         return Preferences.instance
 
     @classmethod
-    def _reset(cls):
+    def _reset(cls) -> None:
         if hasattr(cls, "instance"):
             del cls.instance
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Creates a preference class that will automatically read the file in
         a different thread. Any call to its methods will be blocked until the
         thread is finished reading.
         """
         self.table = NetworkTables.getTable(self.TABLE_NAME)
-        self.table.addTableListenerEx(self.valueChangedEx, NetworkTables.NotifyFlags.NEW | NetworkTables.NotifyFlags.IMMEDIATE)
+        self.table.addTableListenerEx(
+            self.valueChangedEx,
+            NetworkTables.NotifyFlags.NEW | NetworkTables.NotifyFlags.IMMEDIATE,
+        )
 
         hal.report(hal.UsageReporting.kResourceType_Preferences, 0)
 
-    def getKeys(self):
+    def getKeys(self) -> List[str]:
         """:returns: a list of the keys
         """
         with self.lock:
             return [x for x in self.table.getKeys()]
 
-    def keys(self):
+    def keys(self) -> List[str]:
         """Python style get list of keys.
         """
         with self.lock:
             return [x for x in self.table.getKeys()]
 
-    def putString(self, key, value):
+    def putString(self, key: str, value: str) -> None:
         """Puts the given string into the preferences table.
 
         The value may not have quotation marks, nor may the key have any
@@ -91,11 +95,13 @@ class Preferences:
         :param value: the value
         """
         if '"' in value:
-            raise ValueError("Can not put string: '%s' because it contains quotation marks" % value)
+            raise ValueError(
+                "Can not put string: '%s' because it contains quotation marks" % value
+            )
         self.table.putString(key, value)
         self.table.setPersistent(key)
 
-    def putInt(self, key, value):
+    def putInt(self, key: str, value: int) -> None:
         """Puts the given int into the preferences table.
 
         The key may not have any whitespace nor an equals sign.
@@ -110,7 +116,7 @@ class Preferences:
         self.table.putNumber(key, value)
         self.table.setPersistent(key)
 
-    def putFloat(self, key, value):
+    def putFloat(self, key: str, value: float) -> None:
         """Puts the given float into the preferences table.
 
         The key may not have any whitespace nor an equals sign.
@@ -125,7 +131,7 @@ class Preferences:
         self.table.putNumber(key, value)
         self.table.setPersistent(key)
 
-    def putBoolean(self, key, value):
+    def putBoolean(self, key: str, value: bool) -> None:
         """Puts the given float into the preferences table.
 
         The key may not have any whitespace nor an equals sign.
@@ -144,7 +150,7 @@ class Preferences:
         """Python style setting of key/value."""
         self.table.putString(key, str(value))
 
-    def containsKey(self, key):
+    def containsKey(self, key: str) -> bool:
         """Returns whether or not there is a key with the given name.
 
         :param key: the key
@@ -156,7 +162,7 @@ class Preferences:
         """Python style contains key."""
         return self.table.containsKey(key)
 
-    def remove(self, key):
+    def remove(self, key: str) -> None:
         """Remove a preference
 
         :param key: the key
@@ -168,7 +174,7 @@ class Preferences:
         """
         self.table.delete(key)
 
-    def getString(self, key, backup=None):
+    def getString(self, key: str, backup: Optional[str] = None) -> str:
         """Returns the string at the given key. If this table does not have a
         value for that position, then the given backup value will be returned.
 
@@ -178,7 +184,7 @@ class Preferences:
         """
         return self.table.getString(key, backup)
 
-    def getInt(self, key, backup=None):
+    def getInt(self, key: str, backup: Optional[int] = None) -> int:
         """Returns the int at the given key. If this table does not have a
         value for that position, then the given backup value will be returned.
 
@@ -189,7 +195,7 @@ class Preferences:
         """
         return self.table.getNumber(key, backup)
 
-    def getFloat(self, key, backup=None):
+    def getFloat(self, key: str, backup: Optional[float] = None) -> float:
         """Returns the float at the given key. If this table does not have a
         value for that position, then the given backup value will be returned.
 
@@ -200,7 +206,7 @@ class Preferences:
         """
         return self.table.getNumber(key, backup)
 
-    def getBoolean(self, key, backup=None):
+    def getBoolean(self, key: str, backup: Optional[bool] = None) -> bool:
         """Returns the boolean at the given key. If this table does not have a
         value for that position, then the given backup value will be returned.
 
@@ -210,5 +216,5 @@ class Preferences:
         """
         return self.table.getBoolean(key, backup)
 
-    def valueChangedEx(self, source, key, value, isNew):
+    def valueChangedEx(self, source, key, value, isNew) -> None:
         self.table.setPersistent(key)
