@@ -1,25 +1,31 @@
-# validated: 2017-12-15 EN f9bece2ffbf7 edu/wpi/first/wpilibj/PWM.java
-#----------------------------------------------------------------------------
+# validated: 2018-09-09 EN 0614913f1abb edu/wpi/first/wpilibj/PWM.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2014. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+import enum
+from typing import Tuple
 
 import hal
 import weakref
 
+from .motorsafety import MotorSafety
 from .sendablebase import SendableBase
 from .resource import Resource
-from .sensorbase import SensorBase
+from .sensorutil import SensorUtil
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["PWM"]
 
-def _freePWM(handle):
+
+def _freePWM(handle: hal.DigitalHandle) -> None:
     hal.setPWMDisabled(handle)
     hal.freePWMPort(handle)
 
-class PWM(SendableBase):
+
+class PWM(MotorSafety, SendableBase):
     """Raw interface to PWM generation in the FPGA.
     
     The values supplied as arguments for PWM outputs range from -1.0 to 1.0. They
@@ -52,73 +58,88 @@ class PWM(SendableBase):
       controllers. Due to the shipping firmware on the Jaguar, we can't run the
       update period less than 5.05 ms.
     """
-    class PeriodMultiplier:
+
+    class PeriodMultiplier(enum.IntEnum):
         """Represents the amount to multiply the minimum servo-pulse pwm
         period by.
         """
-        
+
         #: Period Multiplier: don't skip pulses. PWM pulses occur every 5.005 ms
         k1X = 1
-        
+
         #: Period Multiplier: skip every other pulse. PWM pulses occur every 10.010 ms
         k2X = 2
-        
+
         #: Period Multiplier: skip three out of four pulses. PWM pulses occur every 20.020 ms
         k4X = 4
-    
-    def __init__(self, channel):
+
+    def __init__(self, channel: int) -> None:
         """Allocate a PWM given a channel.
 
         :param channel: The PWM channel number. 0-9 are on-board, 10-19 are on the MXP port
-        :type channel: int
         """
         super().__init__()
-        SensorBase.checkPWMChannel(channel)
+        SendableBase.__init__(self)
+
+        SensorUtil.checkPWMChannel(channel)
         self.channel = channel
-        
+
         self._handle = hal.initializePWMPort(hal.getPort(channel))
         self.__finalizer = weakref.finalize(self, _freePWM, self._handle)
 
         self.setDisabled()
-        
+
         hal.setPWMEliminateDeadband(self.handle, False)
-        
+
         hal.report(hal.UsageReporting.kResourceType_PWM, channel)
         self.setName("PWM", channel)
-                
+
+        self.setSafetyEnabled(False)
+
         # Python-specific: Need this to free on unit test wpilib reset
         Resource._add_global_resource(self)
-        
-    
+
     @property
-    def handle(self):
+    def handle(self) -> hal.DigitalHandle:
         if not self.__finalizer.alive:
             raise ValueError("Cannot use channel after free() has been called")
         return self._handle
 
-    def free(self):
+    def close(self) -> None:
         """Free the PWM channel.
 
         Free the resource associated with the PWM channel and set the value
         to 0.
         """
-        super().free()
+        super().close()
         if self._handle is None:
             return
         self.__finalizer()
         self._handle = None
 
-    def enableDeadbandElimination(self, eliminateDeadband):
+    def stopMotor(self):
+        self.setDisabled()
+
+    def getDescription(self):
+        return "PWM %d" % self.getChannel()
+
+    def enableDeadbandElimination(self, eliminateDeadband: bool) -> None:
         """Optionally eliminate the deadband from a speed controller.
 
         :param eliminateDeadband: If True, set the motor curve on the Jaguar
             to eliminate the deadband in the middle of the range. Otherwise, keep
             the full range without modifying any values.
-        :type eliminateDeadband: bool
         """
         hal.setPWMEliminateDeadband(self.handle, eliminateDeadband)
 
-    def setBounds(self, max, deadbandMax, center, deadbandMin, min):
+    def setBounds(
+        self,
+        max: float,
+        deadbandMax: float,
+        center: float,
+        deadbandMin: float,
+        min: float,
+    ) -> None:
         """Set the bounds on the PWM pulse widths.
 
         This sets the bounds on the PWM values for a particular type of
@@ -126,19 +147,14 @@ class PWM(SendableBase):
         as the deadband bracket.
 
         :param max: The max PWM pulse width in ms
-        :type max: float
         :param deadbandMax: The high end of the deadband range pulse width in ms
-        :type deadbandMax: float
         :param center: The center (off) pulse width in ms
-        :type center: float
         :param deadbandMin: The low end of the deadband pulse width in ms
-        :type deadbandMin: float
         :param min: The minimum pulse width in ms
-        :type min: float
         """
         hal.setPWMConfig(self.handle, max, deadbandMax, center, deadbandMin, min)
-    
-    def getRawBounds(self):
+
+    def getRawBounds(self) -> Tuple[int, int, int, int, int]:
         """Gets the bounds on the PWM pulse widths. This Gets the bounds on the PWM values for a
         particular type of controller. The values determine the upper and lower speeds as well
         as the deadband bracket.
@@ -147,15 +163,14 @@ class PWM(SendableBase):
         """
         return hal.getPWMConfigRaw(self.handle)
 
-    def getChannel(self):
+    def getChannel(self) -> int:
         """Gets the channel number associated with the PWM Object.
 
         :returns: The channel number.
-        :rtype: int
         """
         return self.channel
 
-    def setPosition(self, pos):
+    def setPosition(self, pos: float) -> None:
         """Set the PWM value based on a position.
 
         This is intended to be used by servos.
@@ -165,11 +180,10 @@ class PWM(SendableBase):
             :func:`setBounds` must be called first.
 
         :param pos: The position to set the servo between 0.0 and 1.0.
-        :type pos: float
         """
         hal.setPWMPosition(self.handle, pos)
 
-    def getPosition(self):
+    def getPosition(self) -> float:
         """Get the PWM value in terms of a position.
 
         This is intended to be used by servos.
@@ -179,11 +193,10 @@ class PWM(SendableBase):
             :func:`setBounds` must be called first.
 
         :returns: The position the servo is set to between 0.0 and 1.0.
-        :rtype: float
         """
         return hal.getPWMPosition(self.handle)
 
-    def setSpeed(self, speed):
+    def setSpeed(self, speed: float) -> None:
         """Set the PWM value based on a speed.
 
         This is intended to be used by speed controllers.
@@ -194,11 +207,10 @@ class PWM(SendableBase):
 
         :param speed: The speed to set the speed controller between -1.0 and
             1.0.
-        :type speed: float
         """
         hal.setPWMSpeed(self.handle, speed)
 
-    def getSpeed(self):
+    def getSpeed(self) -> float:
         """Get the PWM value in terms of speed.
 
         This is intended to be used by speed controllers.
@@ -208,41 +220,37 @@ class PWM(SendableBase):
             :func:`setBounds` must be called first.
 
         :returns: The most recently set speed between -1.0 and 1.0.
-        :rtype: float
         """
         return hal.getPWMSpeed(self.handle)
 
-    def setRaw(self, value):
+    def setRaw(self, value: int) -> None:
         """Set the PWM value directly to the hardware.
 
         Write a raw value to a PWM channel.
 
         :param value: Raw PWM value.  Range 0 - 255.
-        :type value: int
         """
         hal.setPWMRaw(self.handle, value)
 
-    def getRaw(self):
+    def getRaw(self) -> int:
         """Get the PWM value directly from the hardware.
 
         Read a raw value from a PWM channel.
 
         :returns: Raw PWM control value.  Range: 0 - 255.
-        :rtype: int
         """
         return hal.getPWMRaw(self.handle)
-    
-    def setDisabled(self):
+
+    def setDisabled(self) -> None:
         """Temporarily disables the PWM output. The next set call will reenable
         the output.
         """
         hal.setPWMDisabled(self.handle)
 
-    def setPeriodMultiplier(self, mult):
+    def setPeriodMultiplier(self, mult: PeriodMultiplier) -> None:
         """Slow down the PWM signal for old devices.
 
         :param mult: The period multiplier to apply to this channel
-        :type mult: PWM.PeriodMultiplier
         """
         if mult == PWM.PeriodMultiplier.k4X:
             # Squelch 3 out of 4 outputs
@@ -256,10 +264,12 @@ class PWM(SendableBase):
         else:
             raise ValueError("Invalid mult argument '%s'" % mult)
 
-    def setZeroLatch(self):
+    def setZeroLatch(self) -> None:
         hal.latchPWMZero(self.handle)
 
-    def initSendable(self, builder):
-        builder.setSmartDashboardType("Speed Controller")
+    def initSendable(self, builder: SendableBuilder) -> None:
+        builder.setSmartDashboardType("PWM")
         builder.setSafeState(self.setDisabled)
-        builder.addDoubleProperty("Value", self.getSpeed, self.setSpeed)
+        builder.addDoubleProperty(
+            "Value", self.getRaw, lambda value: self.setRaw(int(value))
+        )
