@@ -1,15 +1,17 @@
 # validated: 2017-11-21 EN b65447b6f5a8 edu/wpi/first/wpilibj/Resource.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import weakref
+from typing import Optional
 
 __all__ = ["Resource"]
 
+
 class Resource:
     """Tracks resources in the program.
     
@@ -22,42 +24,45 @@ class Resource:
 
     .. not_implemented: restartProgram
     """
-    
+
     _resource_objects = []
     _global_resources = []
-    
+
     @staticmethod
-    def _reset():
-        '''
+    def _reset() -> None:
+        """
             This clears all resources in the program and calls free() on any
             objects that have a free method.
-        '''
-        
+        """
+
         for resource in Resource._resource_objects:
-            
+
             # free all the resources, if a free method is defined
             for ref in resource.numAllocated:
                 if ref is None:
                     continue
                 obj = ref()
-                if obj is not None and hasattr(obj, 'free'):
-                    obj.free()
-            
-            resource.numAllocated = [None]*len(resource.numAllocated)
-        
+                if obj is not None:
+                    if hasattr(obj, "close"):
+                        obj.close()
+                    elif hasattr(obj, "free"):
+                        obj.free()
+
+            resource.numAllocated = [None] * len(resource.numAllocated)
         for ref in Resource._global_resources:
             obj = ref()
-            if obj is not None and hasattr(obj, 'free'):
-                obj.free()
-        
+            if obj is not None:
+                if hasattr(obj, "close"):
+                    obj.close()
+                elif hasattr(obj, "free"):
+                    obj.free()
         del Resource._global_resources[:]
 
     @staticmethod
-    def _add_global_resource(obj):
+    def _add_global_resource(obj: object) -> None:
         Resource._global_resources.append(weakref.ref(obj))
-        
 
-    def __init__(self, size):
+    def __init__(self, size: int) -> None:
         """Allocate storage for a new instance of Resource.
         Allocate a bool array of values that will get initialized to
         indicate that no resources have been allocated yet. The indices
@@ -66,9 +71,9 @@ class Resource:
         :param size: The number of blocks to allocate
         """
         Resource._resource_objects.append(self)
-        self.numAllocated = [None]*size
+        self.numAllocated = [None] * size
 
-    def allocate(self, obj, index=None):
+    def allocate(self, obj: object, index: Optional[int] = None) -> int:
         """Allocate a resource.
 
         When index is None or unspecified, a free resource value within the
@@ -97,7 +102,7 @@ class Resource:
         self.numAllocated[index] = weakref.ref(obj)
         return index
 
-    def free(self, index):
+    def free(self, index: int) -> None:
         """Force-free an allocated resource.
         After a resource is no longer needed, for example a destructor is
         called for a channel assignment class, free will release the resource
