@@ -1,19 +1,22 @@
-# validated: 2017-12-06 DV dd7563376bf6 edu/wpi/first/wpilibj/RobotBase.java
-#----------------------------------------------------------------------------
-# Copyright (c) FIRST 2008-2012. All Rights Reserved.
+# validated: 2018-12-31 TW d817001259d6 edu/wpi/first/wpilibj/RobotBase.java
+# ----------------------------------------------------------------------------
+# Copyright (c) FIRST 2008-2018. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+from typing import Type
 
 import hal
 from networktables import NetworkTables
 
 import logging
-logger = logging.getLogger('robotpy')
+
+logger = logging.getLogger("robotpy")
 
 __all__ = ["RobotBase"]
 
+
 class RobotBase:
     """Implement a Robot Program framework.
 
@@ -35,7 +38,7 @@ class RobotBase:
     .. not_implemented: getBooleanProperty
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         NetworkTables.setNetworkIdentity("Robot")
 
         if self.isReal():
@@ -44,16 +47,30 @@ class RobotBase:
             NetworkTables.startServer()
 
         from .driverstation import DriverStation
+
         self.ds = DriverStation.getInstance()
 
-        NetworkTables.getTable("LiveWindow").getSubTable(".status").getEntry("LW Enabled").setBoolean(False)
+        NetworkTables.getTable("LiveWindow").getSubTable(".status").getEntry(
+            "LW Enabled"
+        ).setBoolean(False)
         from .livewindow import LiveWindow
+
         LiveWindow.setEnabled(False)
 
+        from .shuffleboard.shuffleboard import Shuffleboard
+
+        Shuffleboard.disableActuatorWidgets()
+
         self.__initialized = True
 
-    def free(self):
-        """Free the resources for a RobotBase class."""
+    def free(self) -> None:
+        """Free the resources for a RobotBase class.
+
+        .. deprecated:: 2019.0.0
+            Use :meth:`close` instead"""
+        pass
+
+    def close(self) -> None:
         pass
 
     @staticmethod
@@ -118,67 +135,77 @@ class RobotBase:
         """
         return self.ds.isNewControlData()
 
-    def startCompetition(self):
+    def startCompetition(self) -> None:
         """Provide an alternate "main loop" via startCompetition()."""
         raise NotImplementedError
 
     @staticmethod
-    def initializeHardwareConfiguration():
-        """Common initialization for all robot programs."""
-
-        # Python specific: do not call this, initialize() is already called when
-        # hal is imported
-        #hal.initialize()
-
-        from .driverstation import DriverStation
-        from .robotstate import RobotState
-        RobotState.impl = DriverStation.getInstance()
-
-    @staticmethod
-    def main(robot_cls):
+    def main(robot_cls: Type["RobotBase"]) -> bool:
         """Starting point for the applications."""
-        RobotBase.initializeHardwareConfiguration()
-
-        hal.report(hal.UsageReporting.kResourceType_Language,
-                   hal.UsageReporting.kLanguage_Python)
+        # Python-specific: don't call hal.initialize() again here.
+        hal.report(
+            hal.UsageReporting.kResourceType_Language,
+            hal.UsageReporting.kLanguage_Python,
+        )
 
         try:
             robot = robot_cls()
         except:
             from .driverstation import DriverStation
-            DriverStation.reportError("Unhandled exception instantiating robot " + robot_cls.__name__, True)
+
+            DriverStation.reportError(
+                "Unhandled exception instantiating robot " + robot_cls.__name__, True
+            )
             DriverStation.reportWarning("Robots should not quit, but yours did!", False)
-            DriverStation.reportError("Could not instantiate robot "+robot_cls.__name__+"!", False)
+            DriverStation.reportError(
+                "Could not instantiate robot " + robot_cls.__name__ + "!", False
+            )
             return False
 
         # Add a check to see if the user forgot to call super().__init__()
-        if not hasattr(robot, '_RobotBase__initialized'):
-            logger.error("If your robot class has an __init__ function, it must call super().__init__()!")
+        if not hasattr(robot, "_RobotBase__initialized"):
+            logger.error(
+                "If your robot class has an __init__ function, it must call super().__init__()!"
+            )
             return False
 
         if not hal.isSimulation():
             try:
                 import wpilib
-                with open('/tmp/frc_versions/FRC_Lib_Version.ini', 'w') as fp:
-                    fp.write('RobotPy %s' % wpilib.__version__)
+
+                with open("/tmp/frc_versions/FRC_Lib_Version.ini", "w") as fp:
+                    fp.write("RobotPy %s" % wpilib.__version__)
             except:
-                logger.warning("Could not write FRC version file to disk")
+                from .driverstation import DriverStation
+
+                DriverStation.reportError(
+                    "Could not write FRC version file to disk", True
+                )
 
         try:
             robot.startCompetition()
         except KeyboardInterrupt:
-            logger.exception("THIS IS NOT AN ERROR: The user hit CTRL-C to kill the robot")
+            logger.exception(
+                "THIS IS NOT AN ERROR: The user hit CTRL-C to kill the robot"
+            )
             logger.info("Exiting because of keyboard interrupt")
             return True
         except:
             from .driverstation import DriverStation
+
             DriverStation.reportError("Unhandled exception", True)
             DriverStation.reportWarning("Robots should not quit, but yours did!", False)
-            DriverStation.reportError("The startCompetition() method (or methods called by it) should have handled the exception above.", False)
+            DriverStation.reportError(
+                "The startCompetition() method (or methods called by it) should have handled the exception above.",
+                False,
+            )
             return False
         else:
             # startCompetition never returns unless exception occurs....
             from .driverstation import DriverStation
+
             DriverStation.reportWarning("Robots should not quit, but yours did!", False)
-            DriverStation.reportError("Unexpected return from startCompetition() method.", False)
+            DriverStation.reportError(
+                "Unexpected return from startCompetition() method.", False
+            )
             return False
