@@ -5,6 +5,7 @@
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
 # ----------------------------------------------------------------------------
+from typing import List, Tuple
 
 import hal
 import math
@@ -17,12 +18,15 @@ from .talon import Talon
 __all__ = ["RobotDrive"]
 
 
-def _freeRobotDrive(allocatedSpeedControllers):
+def _freeRobotDrive(allocatedSpeedControllers) -> None:
     """
     Free the speed controllers if they were allocated locally
     """
     for sc in allocatedSpeedControllers:
-        if hasattr(sc, "free"):
+        sc_close = getattr(sc, "close", None)
+        if sc_close is not None:
+            sc_close()
+        elif hasattr(sc, "free"):
             sc.free()
 
 
@@ -32,7 +36,7 @@ class RobotDrive(MotorSafety):
         Use :class:`.DifferentialDrive` or :class:`.MecanumDrive` instead.
 
     Operations on a robot drivetrain based on a definition of the motor
-    configuration. 
+    configuration.
 
     The robot drive class handles basic driving for a robot. Currently, 2
     and 4 motor tank and mecanum drive trains are supported. In the future
@@ -72,7 +76,7 @@ class RobotDrive(MotorSafety):
     kMecanumCartesian_Reported = False
     kMecanumPolar_Reported = False
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         """Constructor for RobotDrive.
 
         Either 2 or 4 motors can be passed to the constructor to implement
@@ -92,7 +96,9 @@ class RobotDrive(MotorSafety):
         """
         warnings.warn(
             "RobotDrive was deprecated in 2018.0.0. Use DifferentialDrive, KilloughDrive, or MecanumDrive instead.",
-            DeprecationWarning, stacklevel=2)
+            DeprecationWarning,
+            stacklevel=2,
+        )
         super().__init__()
 
         # keyword arguments
@@ -111,8 +117,9 @@ class RobotDrive(MotorSafety):
             controllerClass = Talon
 
         if kwargs:
-            warnings.warn("unknown keyword arguments: %s" % kwargs.keys(),
-                          RuntimeWarning)
+            warnings.warn(
+                "unknown keyword arguments: %s" % kwargs.keys(), RuntimeWarning
+            )
 
         # positional arguments
         if len(args) == 2:
@@ -124,25 +131,25 @@ class RobotDrive(MotorSafety):
             self.frontRightMotor = args[2]
             self.rearRightMotor = args[3]
         elif len(args) != 0:
-            raise ValueError("don't know how to handle %d positional arguments" % len(args))
+            raise ValueError(
+                "don't know how to handle %d positional arguments" % len(args)
+            )
 
         self.allocatedSpeedControllers = list()
 
         # convert channel number into motor controller if needed
-        if (self.frontLeftMotor is not None and
-                not hasattr(self.frontLeftMotor, "set")):
+        if self.frontLeftMotor is not None and not hasattr(self.frontLeftMotor, "set"):
             self.frontLeftMotor = controllerClass(self.frontLeftMotor)
             self.allocatedSpeedControllers.append(self.frontLeftMotor)
-        if (self.rearLeftMotor is not None and
-                not hasattr(self.rearLeftMotor, "set")):
+        if self.rearLeftMotor is not None and not hasattr(self.rearLeftMotor, "set"):
             self.rearLeftMotor = controllerClass(self.rearLeftMotor)
             self.allocatedSpeedControllers.append(self.rearLeftMotor)
-        if (self.frontRightMotor is not None and
-                not hasattr(self.frontRightMotor, "set")):
+        if self.frontRightMotor is not None and not hasattr(
+            self.frontRightMotor, "set"
+        ):
             self.frontRightMotor = controllerClass(self.frontRightMotor)
             self.allocatedSpeedControllers.append(self.frontRightMotor)
-        if (self.rearRightMotor is not None and
-                not hasattr(self.rearRightMotor, "set")):
+        if self.rearRightMotor is not None and not hasattr(self.rearRightMotor, "set"):
             self.rearRightMotor = controllerClass(self.rearRightMotor)
             self.allocatedSpeedControllers.append(self.rearRightMotor)
 
@@ -155,12 +162,14 @@ class RobotDrive(MotorSafety):
         self.setSafetyEnabled(True)
 
         # Setup Finalizer
-        self.__finalizer = weakref.finalize(self, _freeRobotDrive, self.allocatedSpeedControllers)
+        self.__finalizer = weakref.finalize(
+            self, _freeRobotDrive, self.allocatedSpeedControllers
+        )
 
         # start off not moving
         self.drive(0, 0)
 
-    def drive(self, outputMagnitude, curve):
+    def drive(self, outputMagnitude: float, curve: float) -> None:
         """Drive the motors at "outputMagnitude" and "curve".
 
         Both outputMagnitude and curve are -1.0 to +1.0 values, where 0.0
@@ -183,23 +192,25 @@ class RobotDrive(MotorSafety):
         wheelbase w.
         """
         if not RobotDrive.kArcadeRatioCurve_Reported:
-            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
-                       self.getNumMotors(),
-                       hal.UsageReporting.kRobotDrive_ArcadeRatioCurve)
+            hal.report(
+                hal.UsageReporting.kResourceType_RobotDrive,
+                self.getNumMotors(),
+                hal.UsageReporting.kRobotDrive_ArcadeRatioCurve,
+            )
             RobotDrive.kArcadeRatioCurve_Reported = True
 
         if curve < 0:
             value = math.log(-curve)
             ratio = (value - self.sensitivity) / (value + self.sensitivity)
             if ratio == 0:
-                ratio = .0000000001
+                ratio = 0.0000000001
             leftOutput = outputMagnitude / ratio
             rightOutput = outputMagnitude
         elif curve > 0:
             value = math.log(curve)
             ratio = (value - self.sensitivity) / (value + self.sensitivity)
             if ratio == 0:
-                ratio = .0000000001
+                ratio = 0.0000000001
             leftOutput = outputMagnitude
             rightOutput = outputMagnitude / ratio
         else:
@@ -208,7 +219,7 @@ class RobotDrive(MotorSafety):
 
         self.setLeftRightMotorOutputs(leftOutput, rightOutput)
 
-    def tankDrive(self, *args, **kwargs):
+    def tankDrive(self, *args, **kwargs) -> None:
         """Provide tank steering using the stored robot configuration.
 
         Either two joysticks (with optional specified axis) or two raw values
@@ -241,9 +252,11 @@ class RobotDrive(MotorSafety):
             sensitivity at lower speeds.  Defaults to True if unspecified.
         """
         if not RobotDrive.kTank_Reported:
-            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
-                       self.getNumMotors(),
-                       hal.UsageReporting.kRobotDrive_Tank)
+            hal.report(
+                hal.UsageReporting.kResourceType_RobotDrive,
+                self.getNumMotors(),
+                hal.UsageReporting.kRobotDrive_Tank,
+            )
             RobotDrive.kTank_Reported = True
 
         # keyword arguments
@@ -256,8 +269,9 @@ class RobotDrive(MotorSafety):
         squaredInputs = kwargs.pop("squaredInputs", None)
 
         if kwargs:
-            warnings.warn("unknown keyword arguments: %s" % kwargs.keys(),
-                          RuntimeWarning)
+            warnings.warn(
+                "unknown keyword arguments: %s" % kwargs.keys(), RuntimeWarning
+            )
 
         # positional arguments
         if len(args) == 2 or len(args) == 3:
@@ -278,7 +292,9 @@ class RobotDrive(MotorSafety):
         elif len(args) == 5:
             leftStick, leftAxis, rightStick, rightAxis, squaredInputs = args
         elif len(args) != 0:
-            raise ValueError("don't know how to handle %d positional arguments" % len(args))
+            raise ValueError(
+                "don't know how to handle %d positional arguments" % len(args)
+            )
 
         # get value from stick if only stick provided
         if leftValue is None:
@@ -306,7 +322,7 @@ class RobotDrive(MotorSafety):
 
         self.setLeftRightMotorOutputs(leftValue, rightValue)
 
-    def arcadeDrive(self, *args, **kwargs):
+    def arcadeDrive(self, *args, **kwargs) -> None:
         """Provide tank steering using the stored robot configuration.
 
         Either one or two joysticks (with optional specified axis) or two raw
@@ -345,9 +361,11 @@ class RobotDrive(MotorSafety):
         """
 
         if not RobotDrive.kArcadeStandard_Reported:
-            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
-                       self.getNumMotors(),
-                       hal.UsageReporting.kRobotDrive_ArcadeStandard)
+            hal.report(
+                hal.UsageReporting.kResourceType_RobotDrive,
+                self.getNumMotors(),
+                hal.UsageReporting.kRobotDrive_ArcadeStandard,
+            )
             RobotDrive.kArcadeStandard_Reported = True
 
         # keyword arguments
@@ -361,8 +379,9 @@ class RobotDrive(MotorSafety):
         squaredInputs = kwargs.pop("squaredInputs", None)
 
         if kwargs:
-            warnings.warn("unknown keyword arguments: %s" % kwargs.keys(),
-                          RuntimeWarning)
+            warnings.warn(
+                "unknown keyword arguments: %s" % kwargs.keys(), RuntimeWarning
+            )
 
         # positional arguments
         if len(args) == 1:
@@ -380,7 +399,9 @@ class RobotDrive(MotorSafety):
         elif len(args) == 5:
             moveStick, moveAxis, rotateStick, rotateAxis, squaredInputs = args
         elif len(args) != 0:
-            raise ValueError("don't know how to handle %d positional arguments" % len(args))
+            raise ValueError(
+                "don't know how to handle %d positional arguments" % len(args)
+            )
 
         # get value from stick if only stick provided
         if moveValue is None:
@@ -425,7 +446,9 @@ class RobotDrive(MotorSafety):
 
         self.setLeftRightMotorOutputs(leftMotorSpeed, rightMotorSpeed)
 
-    def mecanumDrive_Cartesian(self, x, y, rotation, gyroAngle):
+    def mecanumDrive_Cartesian(
+        self, x: float, y: float, rotation: float, gyroAngle: float
+    ) -> None:
         """Drive method for Mecanum wheeled robots.
 
         A method for driving with Mecanum wheeled robots. There are 4 wheels
@@ -446,9 +469,11 @@ class RobotDrive(MotorSafety):
             to implement field-oriented controls.
         """
         if not RobotDrive.kMecanumCartesian_Reported:
-            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
-                       self.getNumMotors(),
-                       hal.UsageReporting.kRobotDrive_MecanumCartesian)
+            hal.report(
+                hal.UsageReporting.kResourceType_RobotDrive,
+                self.getNumMotors(),
+                hal.UsageReporting.kRobotDrive_MecanumCartesian,
+            )
             RobotDrive.kMecanumCartesian_Reported = True
 
         xIn = x
@@ -467,13 +492,17 @@ class RobotDrive(MotorSafety):
         RobotDrive.normalize(wheelSpeeds)
 
         self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.maxOutput)
-        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput)
+        self.frontRightMotor.set(
+            wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput
+        )
         self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.maxOutput)
         self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.maxOutput)
 
         self.feed()
 
-    def mecanumDrive_Polar(self, magnitude, direction, rotation):
+    def mecanumDrive_Polar(
+        self, magnitude: float, direction: float, rotation: float
+    ) -> None:
         """Drive method for Mecanum wheeled robots.
 
         A method for driving with Mecanum wheeled robots. There are 4 wheels
@@ -489,9 +518,11 @@ class RobotDrive(MotorSafety):
             independent of the magnitute or direction. [-1.0..1.0]
         """
         if not RobotDrive.kMecanumPolar_Reported:
-            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
-                       self.getNumMotors(),
-                       hal.UsageReporting.kRobotDrive_MecanumPolar)
+            hal.report(
+                hal.UsageReporting.kResourceType_RobotDrive,
+                self.getNumMotors(),
+                hal.UsageReporting.kRobotDrive_MecanumPolar,
+            )
             RobotDrive.kMecanumPolar_Reported = True
 
         # Normalized for full power along the Cartesian axes.
@@ -510,13 +541,17 @@ class RobotDrive(MotorSafety):
         RobotDrive.normalize(wheelSpeeds)
 
         self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.maxOutput)
-        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput)
+        self.frontRightMotor.set(
+            wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput
+        )
         self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.maxOutput)
         self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.maxOutput)
 
         self.feed()
 
-    def holonomicDrive(self, magnitude, direction, rotation):
+    def holonomicDrive(
+        self, magnitude: float, direction: float, rotation: float
+    ) -> None:
         """Holonomic Drive method for Mecanum wheeled robots.
 
         This is an alias to :func:`mecanumDrive_Polar` for backward
@@ -531,7 +566,7 @@ class RobotDrive(MotorSafety):
         """
         self.mecanumDrive_Polar(magnitude, direction, rotation)
 
-    def setLeftRightMotorOutputs(self, leftOutput, rightOutput):
+    def setLeftRightMotorOutputs(self, leftOutput: float, rightOutput: float) -> None:
         """Set the speed of the right and left motors.
 
         This is used once an appropriate drive setup function is called such as
@@ -558,7 +593,7 @@ class RobotDrive(MotorSafety):
         self.feed()
 
     @staticmethod
-    def limit(number):
+    def limit(number: float) -> float:
         """Limit motor values to the -1.0 to +1.0 range."""
         if number > 1.0:
             return 1.0
@@ -567,7 +602,7 @@ class RobotDrive(MotorSafety):
         return number
 
     @staticmethod
-    def normalize(wheelSpeeds):
+    def normalize(wheelSpeeds: List[float]) -> None:
         """Normalize all wheel speeds if the magnitude of any wheel is greater
         than 1.0.
         """
@@ -577,14 +612,14 @@ class RobotDrive(MotorSafety):
                 wheelSpeeds[i] = wheelSpeeds[i] / maxMagnitude
 
     @staticmethod
-    def rotateVector(x, y, angle):
+    def rotateVector(x: float, y: float, angle: float) -> Tuple[float, float]:
         """Rotate a vector in Cartesian space."""
         angle = math.radians(angle)
         cosA = math.cos(angle)
         sinA = math.sin(angle)
         return (x * cosA - y * sinA), (x * sinA + y * cosA)
 
-    def setInvertedMotor(self, motor, isInverted):
+    def setInvertedMotor(self, motor: int, isInverted: bool) -> bool:
         """Invert a motor direction.
 
         This is used when a motor should run in the opposite direction as
@@ -606,7 +641,7 @@ class RobotDrive(MotorSafety):
         else:
             raise ValueError("Invalid motor type specified")
 
-    def setSensitivity(self, sensitivity):
+    def setSensitivity(self, sensitivity: float) -> None:
         """Set the turning sensitivity.
 
         This only impacts the drive() entry-point.
@@ -616,7 +651,7 @@ class RobotDrive(MotorSafety):
         """
         self.sensitivity = sensitivity
 
-    def setMaxOutput(self, maxOutput):
+    def setMaxOutput(self, maxOutput: float) -> None:
         """Configure the scaling factor for using RobotDrive with motor
         controllers in a mode other than PercentVbus.
 
@@ -625,7 +660,7 @@ class RobotDrive(MotorSafety):
         """
         self.maxOutput = maxOutput
 
-    def free(self):
+    def free(self) -> None:
         self.__finalizer()
         self.frontLeftMotor = None
         self.frontRightMotor = None
@@ -634,10 +669,10 @@ class RobotDrive(MotorSafety):
         self.allocatedSpeedControllers = list()
         self.setSafetyEnabled(False)
 
-    def getDescription(self):
+    def getDescription(self) -> str:
         return "Robot Drive"
 
-    def stopMotor(self):
+    def stopMotor(self) -> None:
         if self.frontLeftMotor is not None:
             self.frontLeftMotor.stopMotor()
         if self.frontRightMotor is not None:
@@ -648,10 +683,14 @@ class RobotDrive(MotorSafety):
             self.rearRightMotor.stopMotor()
         self.feed()
 
-    def getNumMotors(self):
+    def getNumMotors(self) -> int:
         motors = 0
-        if self.frontLeftMotor is not None: motors += 1
-        if self.frontRightMotor is not None: motors += 1
-        if self.rearLeftMotor is not None: motors += 1
-        if self.rearRightMotor is not None: motors += 1
+        if self.frontLeftMotor is not None:
+            motors += 1
+        if self.frontRightMotor is not None:
+            motors += 1
+        if self.rearLeftMotor is not None:
+            motors += 1
+        if self.rearRightMotor is not None:
+            motors += 1
         return motors
