@@ -13,6 +13,7 @@ import hal
 from .livewindow import LiveWindow
 from .robotbase import RobotBase
 from .timer import Timer
+from .shuffleboard import Shuffleboard
 
 __all__ = ["SampleRobot"]
 
@@ -44,7 +45,7 @@ class SampleRobot(RobotBase):
     #: is recommended to use this instead of print statements.
     logger = logging.getLogger("robot")
 
-    def robotInit(self):
+    def robotInit(self) -> None:
         """Robot-wide initialization code should go here.
 
         Users should override this method for default Robot-wide initialization
@@ -59,43 +60,51 @@ class SampleRobot(RobotBase):
                      waits for enable will cause the robot to never indicate that the code is
                      ready, causing the robot to be bypassed in a match.
         """
-        self.logger.info("Default robotInit() method running, consider providing your own")
+        self.logger.info(
+            "Default robotInit() method running, consider providing your own"
+        )
 
-    def disabled(self):
+    def disabled(self) -> None:
         """Disabled should go here.
         Users should overload this method to run code that should run while
         the field is disabled.
 
         Called once each time the robot enters the disabled state.
         """
-        self.logger.info("Default disabled() method running, consider providing your own")
+        self.logger.info(
+            "Default disabled() method running, consider providing your own"
+        )
 
-    def autonomous(self):
+    def autonomous(self) -> None:
         """Autonomous should go here.
         Users should add autonomous code to this method that should run while
         the field is in the autonomous period.
 
         Called once each time the robot enters the autonomous state.
         """
-        self.logger.info("Default autonomous() method running, consider providing your own")
+        self.logger.info(
+            "Default autonomous() method running, consider providing your own"
+        )
 
-    def operatorControl(self):
+    def operatorControl(self) -> None:
         """Operator control (tele-operated) code should go here.
         Users should add Operator Control code to this method that should run
         while the field is in the Operator Control (tele-operated) period.
 
         Called once each time the robot enters the operator-controlled state.
         """
-        self.logger.warning("Default operatorControl() method running, consider providing your own")
+        self.logger.warning(
+            "Default operatorControl() method running, consider providing your own"
+        )
 
-    def test(self):
+    def test(self) -> None:
         """Test code should go here.
         Users should add test code to this method that should run while the
         robot is in test mode.
         """
         self.logger.info("Default test() method running, consider providing your own")
 
-    def robotMain(self):
+    def robotMain(self) -> None:
         """Robot main program for free-form programs.
 
         This should be overridden by user subclasses if the intent is to not
@@ -118,7 +127,7 @@ class SampleRobot(RobotBase):
         """
         self._no_robot_main = True
 
-    def startCompetition(self):
+    def startCompetition(self) -> None:
         """Start a competition.
         This code tracks the order of the field starting to ensure that
         everything happens in the right order. Repeatedly run the correct
@@ -127,8 +136,10 @@ class SampleRobot(RobotBase):
         change, either the other mode starts or the robot is disabled. Then
         go back and wait for the robot to be enabled again.
         """
-        hal.report(hal.UsageReporting.kResourceType_Framework,
-                   hal.UsageReporting.kFramework_Sample)
+        hal.report(
+            hal.UsageReporting.kResourceType_Framework,
+            hal.UsageReporting.kFramework_Sample,
+        )
 
         self.robotInit()
 
@@ -137,7 +148,7 @@ class SampleRobot(RobotBase):
 
         self.robotMain()
 
-        if hasattr(self, '_no_robot_main'):
+        if hasattr(self, "_no_robot_main"):
             while True:
                 if self.isDisabled():
                     self.ds.InDisabled(True)
@@ -153,12 +164,14 @@ class SampleRobot(RobotBase):
                         Timer.delay(0.01)
                 elif self.isTest():
                     LiveWindow.setEnabled(True)
+                    Shuffleboard.enableActuatorWidgets()
                     self.ds.InTest(True)
                     self.test()
                     self.ds.InTest(False)
                     while self.isTest() and self.isEnabled():
                         Timer.delay(0.01)
                     LiveWindow.setEnabled(False)
+                    Shuffleboard.disableActuatorWidgets()
                 else:
                     self.ds.InOperatorControl(True)
                     self.operatorControl()
