@@ -1,15 +1,18 @@
-# validated: 2017-12-16 EN f9bece2ffbf7 edu/wpi/first/wpilibj/smartdashboard/SendableChooser.java
-#----------------------------------------------------------------------------
+# validated: 2018-03-04 DV 9d7293734a65 edu/wpi/first/wpilibj/smartdashboard/SendableChooser.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2017. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+from typing import Any
 
 from .sendablebase import SendableBase
+from .sendablebuilder import SendableBuilder
 
 __all__ = ["SendableChooser"]
 
+
 class SendableChooser(SendableBase):
     """A useful tool for presenting a selection of options to be displayed on
     the :class:`.SmartDashboard`
@@ -43,15 +46,15 @@ class SendableChooser(SendableBase):
     OPTIONS = "options"
     # A table linking strings to the objects the represent
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Instantiates a SendableChooser.
         """
-        super().__init__()
+        super().__init__(addLiveWindow=False)
         self.map = {}
         self.tableSelected = None
         self.defaultChoice = ""
 
-    def addObject(self, name, object):
+    def addObject(self, name: str, object: Any) -> None:
         """Adds the given object to the list of options. On the
         :class:`.SmartDashboard` on the desktop, the object will appear as the
         given name.
@@ -61,7 +64,7 @@ class SendableChooser(SendableBase):
         """
         self.map[name] = object
 
-    def addDefault(self, name, object):
+    def addDefault(self, name: str, object: Any) -> None:
         """Add the given object to the list of options and marks it as the
         default.  Functionally, this is very close to :meth:`.addObject` except
         that it will use this as the default option if none other is
@@ -75,7 +78,7 @@ class SendableChooser(SendableBase):
         self.defaultChoice = name
         self.addObject(name, object)
 
-    def getSelected(self):
+    def getSelected(self) -> Any:
         """Returns the object associated with the selected option. If there
         is none selected, it will return the default. If there is none
         selected and no default, then it will return None.
@@ -87,7 +90,7 @@ class SendableChooser(SendableBase):
             selected = self.tableSelected.getString(self.defaultChoice)
         return self.map.get(selected)
 
-    def initSendable(self, builder):
+    def initSendable(self, builder: SendableBuilder) -> None:
         builder.setSmartDashboardType("String Chooser")
         builder.addStringProperty(self.DEFAULT, lambda: self.defaultChoice, None)
         builder.addStringArrayProperty(self.OPTIONS, lambda: self.map.keys(), None)
