@@ -1,18 +1,26 @@
-# validated: 2018-01-06 DS ee33296e1fe6 edu/wpi/first/wpilibj/smartdashboard/SmartDashboard.java
+# validated: 2018-11-17 EN 89d15f061b33 edu/wpi/first/wpilibj/smartdashboard/SmartDashboard.java
 
 # validation note: 2017-10-22: Not using the getEntry() stuff that Java uses,
 #                              as using the existing table stuff is more
 #                              efficient
 
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2017. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+from typing import List, TypeVar, Union, Tuple, Iterable
+
+T = TypeVar("T")
 
 import hal
 import threading
+
+from networktables.entry import NetworkTableEntry
+from networktables.networktable import NetworkTable
+
+from .sendable import Sendable
 from ._impl.utils import match_arglist, HasAttribute
 from .sendablebuilder import SendableBuilder
 
@@ -20,7 +28,7 @@ __all__ = ["SmartDashboard"]
 
 
 class Data:
-    def __init__(self, sendable):
+    def __init__(self, sendable: Sendable) -> None:
         self.sendable = sendable
         self.builder = SendableBuilder()
 
@@ -41,6 +49,7 @@ class SmartDashboard:
         # sd.putXXX and sd.getXXX work as expected here
     
     """
+
     # The NetworkTable used by SmartDashboard
     table = None
     # A table linking tables in the SmartDashboard to the SmartDashboardData
@@ -49,21 +58,21 @@ class SmartDashboard:
     mutex = threading.RLock()
 
     @classmethod
-    def _reset(cls):
+    def _reset(cls) -> None:
         cls.tablesToData = {}
         cls.table = None
 
     @classmethod
-    def getTable(cls):
+    def getTable(cls) -> NetworkTable:
         if cls.table is None:
             from networktables import NetworkTables
+
             cls.table = NetworkTables.getTable("SmartDashboard")
-            hal.report(hal.UsageReporting.kResourceType_SmartDashboard,
-                       hal.UsageReporting.kSmartDashboard_Instance)
+            hal.report(hal.UsageReporting.kResourceType_SmartDashboard, 0)
         return cls.table
 
     @classmethod
-    def putData(cls, *args, **kwargs):
+    def putData(cls, *args, **kwargs) -> None:
         """
         Maps the specified key (name of the :class:`.Sendable` if not provided) 
         to the specified value in this table.
@@ -76,7 +85,7 @@ class SmartDashboard:
         - value
         
         :param key: the key (cannot be None)
-        :type  key: str
+        :type key: str
         :param data: the value
         :type data: :class:`.Sendable`
         :param value: the value
@@ -86,13 +95,13 @@ class SmartDashboard:
             key_arg = ("key", [str])
             data_arg = ("data", [HasAttribute("initSendable")])
             value_arg = ("value", [HasAttribute("initSendable")])
-            templates = [[key_arg, data_arg],
-                         [value_arg],]
+            templates = [[key_arg, data_arg], [value_arg]]
 
-            index, results = match_arglist('SmartDashboard.putData',
-                                       args, kwargs, templates)
+            index, results = match_arglist(
+                "SmartDashboard.putData", args, kwargs, templates
+            )
             if index == 0:
-                key = results['key']
+                key = results["key"]
                 data = results["data"]
             elif index == 1:
                 data = results["value"]
@@ -112,36 +121,34 @@ class SmartDashboard:
                 data.initSendable(sddata.builder)
                 sddata.builder.updateTable()
                 sddata.builder.startListeners()
-                dataTable.getEntry('.name').setString(key)
+                dataTable.getEntry(".name").setString(key)
 
     @classmethod
-    def getData(cls, key):
+    def getData(cls, key: str) -> Sendable:
         """Returns the value at the specified key.
         
         :param key: the key
-        :type  key: str
-        
+
         :returns: the value
         
         :raises: :exc:`KeyError` if the key doesn't exist
         """
-        data = cls.tablesToData.get(subtable, None)
+        data = cls.tablesToData.get(key, None)
         if data is None:
             raise KeyError("SmartDashboard data does not exist: '%s'" % key)
         return data.sendable
-    
+
     @classmethod
-    def getEntry(cls, key):
+    def getEntry(cls, key: str) -> NetworkTableEntry:
         """Gets the entry for the specified key.
         
         :param key: the key name
-        :rtype: :class:`.NetworkTableEntry`
         """
         table = cls.getTable()
         return table.getEntry(key)
 
     @classmethod
-    def containsKey(cls, key):
+    def containsKey(cls, key: str) -> bool:
         """Checks the table and tells if it contains the specified key.
 
         :param key: key the key to search for
@@ -152,7 +159,7 @@ class SmartDashboard:
         return table.containsKey(key)
 
     @classmethod
-    def getKeys(cls, types=0):
+    def getKeys(cls, types: int = 0) -> List[str]:
         """Get the keys stored in the SmartDashboard table of NetworkTables.
 
         :param types: bitmask of types; 0 is treated as a "don't care".
@@ -163,7 +170,7 @@ class SmartDashboard:
         return table.getKeys(types)
 
     @classmethod
-    def setPersistent(cls, key):
+    def setPersistent(cls, key: str) -> None:
         """Makes a key's value persistent through program restarts.
         The key cannot be null.
 
@@ -173,7 +180,7 @@ class SmartDashboard:
         table.setPersistent(key)
 
     @classmethod
-    def clearPersistent(cls, key):
+    def clearPersistent(cls, key: str) -> None:
         """Stop making a key's value persistent through program restarts.
         The key cannot be null.
 
@@ -183,7 +190,7 @@ class SmartDashboard:
         table.clearPersistent(key)
 
     @classmethod
-    def isPersistent(cls, key):
+    def isPersistent(cls, key: str) -> bool:
         """Returns whether the value is persistent through program restarts.
         The key cannot be null.
 
@@ -195,7 +202,7 @@ class SmartDashboard:
         return table.isPersistent(key)
 
     @classmethod
-    def setFlags(cls, key, flags):
+    def setFlags(cls, key: str, flags: int) -> None:
         """Sets flags on the specified key in this table. The key can
         not be null.
 
@@ -206,7 +213,7 @@ class SmartDashboard:
         table.setFlags(key, flags)
 
     @classmethod
-    def clearFlags(cls, key, flags):
+    def clearFlags(cls, key: str, flags: int) -> None:
         """Clears flags on the specified key in this table. The key can
         not be null.
 
@@ -217,7 +224,7 @@ class SmartDashboard:
         table.clearFlags(key, flags)
 
     @classmethod
-    def getFlags(cls, key):
+    def getFlags(cls, key: str) -> int:
         """ Returns the flags for the specified key.
 
         :param key: the key name
@@ -228,7 +235,7 @@ class SmartDashboard:
         return table.getFlags(key)
 
     @classmethod
-    def delete(cls, key):
+    def delete(cls, key: str) -> None:
         """Deletes the specified key in this table. The key can
         not be null.
 
@@ -238,7 +245,7 @@ class SmartDashboard:
         table.delete(key)
 
     @classmethod
-    def putBoolean(cls, key, value):
+    def putBoolean(cls, key: str, value: bool) -> bool:
         """Put a boolean in the table.
 
         :param key: the key to be assigned to
@@ -250,7 +257,7 @@ class SmartDashboard:
         return table.putBoolean(key, value)
 
     @classmethod
-    def setDefaultBoolean(cls, key, defaultValue):
+    def setDefaultBoolean(cls, key: str, defaultValue: bool) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -262,12 +269,11 @@ class SmartDashboard:
         return table.setDefaultBoolean(key, defaultValue)
 
     @classmethod
-    def getBoolean(cls, key, defaultValue):
+    def getBoolean(cls, key: str, defaultValue: T) -> Union[T, bool]:
         """Returns the boolean the key maps to. If the key does not exist or is of
         different type, it will return the default value.
         
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -277,7 +283,7 @@ class SmartDashboard:
         return table.getBoolean(key, defaultValue)
 
     @classmethod
-    def putNumber(cls, key, value):
+    def putNumber(cls, key: str, value: float) -> bool:
         """Put a number in the table.
         
         :param key: the key to be assigned to
@@ -289,7 +295,7 @@ class SmartDashboard:
         return table.putNumber(key, value)
 
     @classmethod
-    def setDefaultNumber(cls, key, defaultValue):
+    def setDefaultNumber(cls, key: str, defaultValue: float) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -301,12 +307,11 @@ class SmartDashboard:
         return table.setDefaultNumber(key, defaultValue)
 
     @classmethod
-    def getNumber(cls, key, defaultValue):
+    def getNumber(cls, key: str, defaultValue: T) -> Union[T, float]:
         """Returns the number the key maps to. If the key does not exist or is of
         different type, it will return the default value.
         
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -316,7 +321,7 @@ class SmartDashboard:
         return table.getNumber(key, defaultValue)
 
     @classmethod
-    def putString(cls, key, value):
+    def putString(cls, key: str, value: str) -> bool:
         """Put a string in the table.
         
         :param key: the key to be assigned to
@@ -328,7 +333,7 @@ class SmartDashboard:
         return table.putString(key, value)
 
     @classmethod
-    def setDefaultString(cls, key, defaultValue):
+    def setDefaultString(cls, key: str, defaultValue: str) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -340,12 +345,11 @@ class SmartDashboard:
         return table.setDefaultString(key, defaultValue)
 
     @classmethod
-    def getString(cls, key, defaultValue):
+    def getString(cls, key: str, defaultValue: T) -> Union[T, str]:
         """Returns the string the key maps to. If the key does not exist or is of
         different type, it will return the default value.
         
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -355,7 +359,7 @@ class SmartDashboard:
         return table.getString(key, defaultValue)
 
     @classmethod
-    def putBooleanArray(cls, key, value):
+    def putBooleanArray(cls, key: str, value: Iterable[bool]) -> bool:
         """Put a boolean array in the table.
         
         :param key: the key to be assigned to
@@ -367,7 +371,7 @@ class SmartDashboard:
         return table.putBooleanArray(key, value)
 
     @classmethod
-    def setDefaultBooleanArray(cls, key, defaultValue):
+    def setDefaultBooleanArray(cls, key: str, defaultValue: Iterable[bool]) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -379,12 +383,11 @@ class SmartDashboard:
         return table.setDefaultBooleanArray(key, defaultValue)
 
     @classmethod
-    def getBooleanArray(cls, key, defaultValue):
+    def getBooleanArray(cls, key: str, defaultValue: T) -> Union[T, Tuple[bool, ...]]:
         """Returns the boolean array the key maps to. If the key does not exist or is of
         different type, it will return the default value.
         
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -394,7 +397,7 @@ class SmartDashboard:
         return table.getBooleanArray(key, defaultValue)
 
     @classmethod
-    def putNumberArray(cls, key, value):
+    def putNumberArray(cls, key: str, value: Iterable[float]) -> bool:
         """Put a number array in the table.
         
         :param key: the key to be assigned to
@@ -406,7 +409,7 @@ class SmartDashboard:
         return table.putNumberArray(key, value)
 
     @classmethod
-    def setDefaultNumberArray(cls, key, defaultValue):
+    def setDefaultNumberArray(cls, key: str, defaultValue: Iterable[float]) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -418,12 +421,11 @@ class SmartDashboard:
         return table.setDefaultNumberArray(key, defaultValue)
 
     @classmethod
-    def getNumberArray(cls, key, defaultValue):
+    def getNumberArray(cls, key: str, defaultValue: T) -> Union[T, Tuple[float, ...]]:
         """Returns the number array the key maps to. If the key does not exist or is of
         different type, it will return the default value.
 
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -431,57 +433,48 @@ class SmartDashboard:
         """
         table = cls.getTable()
         return table.getNumberArray(key, defaultValue)
-    
+
     @classmethod
-    def putStringArray(cls, key, value):
+    def putStringArray(cls, key: str, value: Iterable[str]) -> bool:
         """Put a string array in the table
         
         :param key: the key to be assigned to
-        :type key: str
         :param value: the value that will be assigned
-        :type value: list(str)
-        
+
         :returns: False if the table key already exists with a different type
-        :rtype: bool
         """
         table = cls.getTable()
         return table.putStringArray(key, value)
-    
+
     @classmethod
-    def setDefaultStringArray(cls, key, defaultValue):
+    def setDefaultStringArray(cls, key: str, defaultValue: Iterable[str]) -> bool:
         """If the key doesn't currently exist, then the specified value will
         be assigned to the key.
         
         :param key: the key to be assigned to
-        :type key: str
         :param defaultValue: the default value to set if key doesn't exist.
-        :type defaultValue: list(str)
-        
+
         :returns: False if the table key exists with a different type
-        :rtype: bool
         """
         table = cls.getTable()
         return table.setDefaultStringArray(key, defaultValue)
-    
+
     @classmethod
-    def getStringArray(cls, key, defaultValue):
+    def getStringArray(cls, key: str, defaultValue: T) -> Union[T, Tuple[str, ...]]:
         """Returns the string array the key maps to. If the key does not exist or is
         of different type, it will return the default value.
         
         :param key: the key to look up
-        :type key: str
         :param defaultValue: the value to be returned if no value is found
-        :type defaultValue: list(str)
-        
+
         :returns: the value associated with the given key or the given default value
                   if there is no value associated with the key
-        :rtype: list(str)
         """
         table = cls.getTable()
         return table.getStringArray(key, defaultValue)
 
     @classmethod
-    def putRaw(cls, key, value):
+    def putRaw(cls, key: str, value: bytes) -> bool:
         """Put a raw value (byte array) in the table.
         
         :param key: the key to be assigned to
@@ -493,7 +486,7 @@ class SmartDashboard:
         return table.putRaw(key, value)
 
     @classmethod
-    def setDefaultRaw(cls, key, defaultValue):
+    def setDefaultRaw(cls, key: str, defaultValue: bytes) -> bool:
         """Gets the current value in the table, setting it if it does not exist.
         
         :param key: the key
@@ -505,12 +498,11 @@ class SmartDashboard:
         return table.setDefaultRaw(key, defaultValue)
 
     @classmethod
-    def getRaw(cls, key, defaultValue):
+    def getRaw(cls, key: str, defaultValue: T) -> Union[T, bytes]:
         """Returns the raw value (byte array) the key maps to. If the key does not exist or is of
         different type, it will return the default value.
 
         :param key: the key to look up
-        :type  key: str
         :param defaultValue: returned if the key doesn't exist
         
         :returns: the value associated with the given key or the given default value
@@ -520,7 +512,7 @@ class SmartDashboard:
         return table.getRaw(key, defaultValue)
 
     @classmethod
-    def updateValues(cls):
+    def updateValues(cls) -> None:
         with cls.mutex:
             for data in cls.tablesToData.values():
                 data.builder.updateTable()
