@@ -1,7 +1,10 @@
-# validated: 2018-01-19 DS a3e5378d1468 edu/wpi/first/wpilibj/SPI.java
+# validated: 2018-09-09 EN 0e9172f9a708 edu/wpi/first/wpilibj/SPI.java
 
 import ctypes
 import enum
+
+from typing import Tuple, Sequence, Union
+
 import hal
 import struct
 import threading
@@ -14,9 +17,11 @@ from .notifier import Notifier
 
 __all__ = ["SPI"]
 
-def _freeSPI(port):
+
+def _freeSPI(port) -> None:
     hal.closeSPI(port)
 
+
 class SPI:
     """Represents a SPI bus port
 
@@ -39,14 +44,13 @@ class SPI:
     devices = 0
 
     @staticmethod
-    def _reset():
+    def _reset() -> None:
         SPI.devices = 0
 
-    def __init__(self, port, simPort=None):
+    def __init__(self, port: Port, simPort=None) -> None:
         """Constructor
 
         :param port: the physical SPI port
-        :type port: :class:`.SPI.Port`
         :param simPort: This must be an object that implements all of
                         the spi* functions from hal_impl that you use.
                         See ``test_spi.py`` for an example.
@@ -59,23 +63,24 @@ class SPI:
             if simPort is None:
                 # If you want more functionality, implement your own mock
                 from hal_impl.spi_helpers import SPISimBase
+
                 simPort = SPISimBase()
 
                 msg = "Using stub simulator for SPI port %s" % port
                 warnings.warn(msg)
 
             # Just check for basic functionality
-            assert hasattr(simPort, 'initializeSPI')
-            assert hasattr(simPort, 'closeSPI')
+            assert hasattr(simPort, "initializeSPI")
+            assert hasattr(simPort, "closeSPI")
 
             self._port = (simPort, port)
         else:
             self._port = port
 
         # python-specific: these are bools instead of integers
-        self.bitOrder = False
-        self.clockPolarity = False
-        self.dataOnTrailing = False
+        self.msbFirst = False
+        self.clockIdleHigh = False
+        self.sampleOnTrailing = False
 
         self.accum = None
 
@@ -86,14 +91,14 @@ class SPI:
         hal.report(hal.UsageReporting.kResourceType_SPI, SPI.devices)
 
     @property
-    def port(self):
+    def port(self) -> Port:
         if not self.__finalizer.alive:
             raise ValueError("Cannot use SPI after free() has been called")
         return self._port
 
-    def free(self):
+    def close(self) -> None:
         if self.accum is not None:
-            self.accum.free()
+            self.accum.close()
             self.accum = None
         self.__finalizer()
 
@@ -109,45 +114,76 @@ class SPI:
         """Configure the order that bits are sent and received on the wire to be most significant bit
         first.
         """
-        self.bitOrder = True
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        self.msbFirst = True
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setLSBFirst(self) -> None:
         """Configure the order that bits are sent and received on the wire to be least significant bit
         first.
         """
-        self.bitOrder = False
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        self.msbFirst = False
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setClockActiveLow(self) -> None:
         """Configure the clock output line to be active low. This is sometimes called clock polarity high
         or clock idle high.
         """
-        self.clockPolarity = True
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        self.clockIdleHigh = True
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setClockActiveHigh(self) -> None:
         """Configure the clock output line to be active high. This is sometimes called clock polarity low
         or clock idle low.
         """
-        self.clockPolarity = False
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        self.clockIdleHigh = False
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
+
+    def setSampleDataOnLeadingEdge(self) -> None:
+        """Configure that the data is stable on the leading edge and the data changes on the trailing edge."""
+        self.sampleOnTrailing = False
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
+
+    def setSampleDataOnTrailingEdge(self) -> None:
+        """Configure that the data is stable on the trailing edge and the data changes on the leading edge."""
+        self.sampleOnTrailing = True
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setSampleDataOnFalling(self) -> None:
-        """Configure that the data is stable on the falling edge and the data changes on the rising edge."""
-        self.dataOnTrailing = True
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        """
+        Configure that the data is stable on the falling edge and the data changes on the rising edge.
+        Note that this gets reversed if setClockActiveLow is set
+
+        .. deprecated:: 2019.0.0
+            Use setSampleDataOnTrailingEdge in most cases
+        """
+        self.sampleOnTrailing = True
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setSampleDataOnRising(self) -> None:
-        """Configure that the data is stable on the rising edge and the data changes on the falling edge."""
-        self.dataOnTrailing = False
-        hal.setSPIOpts(self.port, self.bitOrder, self.dataOnTrailing,
-                       self.clockPolarity)
+        """
+        Configure that the data is stable on the rising edge and the data changes on the falling edge.
+
+        .. deprecated:: 2019.0.0
+            Use setSampleDataOnLeadingEdge in most cases
+        """
+        self.sampleOnTrailing = False
+        hal.setSPIOpts(
+            self.port, self.msbFirst, self.sampleOnTrailing, self.clockIdleHigh
+        )
 
     def setChipSelectActiveHigh(self) -> None:
         """Configure the chip select line to be active high."""
@@ -157,7 +193,7 @@ class SPI:
         """Configure the chip select line to be active low."""
         hal.setSPIChipSelectActiveLow(self.port)
 
-    def write(self, dataToSend: bytes) -> int:
+    def write(self, dataToSend: Union[bytes, Sequence[int]]) -> int:
         """Write data to the slave device.  Blocks until there is space in the
         output FIFO.
 
@@ -165,7 +201,6 @@ class SPI:
         on the MISO input during the transfer into the receive FIFO.
 
         :param dataToSend: Data to send
-        :type dataToSend: iterable of bytes
 
         :returns: Number of bytes written
 
@@ -197,15 +232,14 @@ class SPI:
         :returns: received data bytes
         """
         if initiate:
-            return hal.transactionSPI(self.port, [0]*size)
+            return hal.transactionSPI(self.port, [0] * size)
         else:
             return hal.readSPI(self.port, size)
 
-    def transaction(self, dataToSend: bytes) -> bytes:
+    def transaction(self, dataToSend: Union[bytes, Sequence[int]]) -> bytes:
         """Perform a simultaneous read/write transaction with the device
 
         :param dataToSend: The data to be written out to the device
-        :type dataToSend: iterable of bytes
 
         :returns: data received from the device
 
@@ -254,7 +288,9 @@ class SPI:
         """
         hal.startSPIAutoRate(self.port, period)
 
-    def startAutoTrigger(self, source: DigitalSource, rising: bool, falling: bool) -> None:
+    def startAutoTrigger(
+        self, source: DigitalSource, rising: bool, falling: bool
+    ) -> None:
         """Start running the automatic SPI transfer engine when a trigger occurs.
 
         :meth:`.initAuto` and :meth:`.setAutoTransmitData` must
@@ -264,9 +300,13 @@ class SPI:
         :param rising: trigger on the rising edge
         :param falling: trigger on the falling edge
         """
-        hal.startSPIAutoTrigger(self.port, source.getPortHandleForRouting(),
-                                source.getAnalogTriggerTypeForRouting(),
-                                rising, falling)
+        hal.startSPIAutoTrigger(
+            self.port,
+            source.getPortHandleForRouting(),
+            source.getAnalogTriggerTypeForRouting(),
+            rising,
+            falling,
+        )
 
     def stopAuto(self) -> None:
         """Stop running the automatic SPI transfer engine."""
@@ -276,19 +316,27 @@ class SPI:
         """Force the engine to make a single transfer."""
         hal.forceSPIAutoRead(self.port)
 
-    def readAutoReceivedData(self, buffer, numToRead: int, timeout: float) -> (int, bytes):
+    def readAutoReceivedData(
+        self, buffer: bytes, numToRead: int, timeout: float
+    ) -> Tuple[int, bytes]:
         """Read data that has been transferred by the automatic SPI transfer engine.
 
         Transfers may be made a byte at a time, so it's necessary for the caller
         to handle cases where an entire transfer has not been completed.
 
-        Blocks until numToRead bytes have been read or timeout expires.
-        May be called with numToRead=0 to retrieve how many bytes are available.
+        Each received data sequence consists of a timestamp followed by the
+        received data bytes, one byte per word (in the least significant byte).
+        The length of each received data sequence is the same as the combined
+        size of the data and zeroSize set in setAutoTransmitData().
 
-        :param buffer: A ctypes c_uint8 buffer to read the data into
-        :param numToRead: number of bytes to read
-        :param timeout: timeout in seconds (ms resolution)
-        :returns: Number of bytes remaining to be read
+        Blocks until numToRead words have been read or timeout expires.
+        May be called with numToRead=0 to retrieve how many words are available.
+
+        :param buffer:    A ctypes c_uint32 buffer to read the data into
+        :param numToRead: number of words to read
+        :param timeout:   timeout in seconds (ms resolution)
+        
+        :returns: Number of words remaining to be read
         """
         if len(buffer) < numToRead:
             raise ValueError("buffer is too small, must be at least %s" % numToRead)
@@ -306,28 +354,23 @@ class SPI:
 
         kAccumulateDepth = 2048
 
-        def __init__(self, port: int, xferSize: int, validMask: int,
-                           validValue: int, dataShift: int, dataSize: int,
-                           isSigned: bool, bigEndian: bool):
-
-            # python-specific: for efficiency purposes, we only support xferSize of
-            #                  2, 4, or 8... if you need to do a different size then
-            #                  file a bug and we'll adjust it
-            efmt = '>' if bigEndian else '<'
-            self._xferSize = xferSize                   # SPI transfer size, in bytes
-            if self._xferSize == 2:
-                self._struct = struct.Struct('%sH' % efmt)
-            elif self._xferSize == 4:
-                self._struct = struct.Struct('%sI' % efmt)
-            elif self._xferSize == 8:
-                self._struct = struct.Struct('%sQ' % efmt)
-            else:
-                raise ValueError("SPI Accumulator only supports xferSize of 2/4/8")
-
+        def __init__(
+            self,
+            port: int,
+            xferSize: int,
+            validMask: int,
+            validValue: int,
+            dataShift: int,
+            dataSize: int,
+            isSigned: bool,
+            bigEndian: bool,
+        ) -> None:
             self._mutex = threading.RLock()
             self._notifier = Notifier(self._update)
-            self._buf = (ctypes.c_uint8 * (xferSize * self.kAccumulateDepth))()
+            self._buf = (ctypes.c_uint32 * (xferSize * self.kAccumulateDepth))()
 
+            # fmt: off
+            self._xferSize = xferSize + 1               # +1 for timestamp
             self._validMask = validMask
             self._validValue = validValue
             self._dataShift = dataShift                 # data field shift right amount, in bits
@@ -336,18 +379,30 @@ class SPI:
             self._isSigned = isSigned                   # is data field signed?
             self._bigEndian = bigEndian                 # is response big endian?
             self._port = port
+            # fmt: on
 
             self._value = 0
             self._count = 0
             self._lastValue = 0
+            self._lastTimestamp = 0
+            self._integratedValue = 0.0
 
             self._center = 0
             self._deadband = 0
+            self._integratedCenter = 0.0
 
-        def free(self):
-            self._notifier.stop()
+        def free(self) -> None:
+            """
+            .. deprecated:: 2019.0.0
+                Use close instead
+            """
+            warnings.warn("use close instead", DeprecationWarning, stacklevel=2)
+            self.close()
 
-        def _update(self):
+        def close(self):
+            self._notifier.close()
+
+        def _update(self) -> None:
             with self._mutex:
                 done = False
                 while not done:
@@ -369,13 +424,20 @@ class SPI:
                     hal.readSPIAutoReceivedData(self._port, self._buf, numToRead, 0)
 
                     # loop over all responses
-                    off = 0
-                    while True:
-                        if off > numToRead:
-                            break
+                    for off in range(0, numToRead, self._xferSize):
+                        # get timestamp from first word
+                        timestamp = self._buf[off] & 0xFFFFFFFF
 
                         # convert from bytes
-                        resp = self._struct.unpack_from(self._buf, off)[0]
+                        resp = 0
+                        if self._bigEndian:
+                            for i in range(1, self._xferSize):
+                                resp <<= 8
+                                resp |= self._buf[off + i] & 0xFF
+                        else:
+                            for i in range(self._xferSize - 1, 0):
+                                resp <<= 8
+                                resp |= self._buf[off + i] & 0xFF
 
                         # process response
                         if (resp & self._validMask) == self._validValue:
@@ -385,15 +447,36 @@ class SPI:
                             data &= self._dataMax - 1
 
                             # 2s complement conversion if signed MSB is set
-                            if (self._isSigned and (data & self._dataMsbMask) != 0):
+                            if self._isSigned and (data & self._dataMsbMask) != 0:
                                 data -= self._dataMax
 
                             # center offset
+                            dataNoCenter = data
                             data -= self._center
 
                             # only accumulate if outside deadband
-                            if (data < -self._deadband or data > self._deadband):
+                            if data < -self._deadband or data > self._deadband:
                                 self._value += data
+                                if self._count != 0:
+                                    # timestamps use the 1us FPGA clock; also handle rollover
+                                    if timestamp >= self._lastTimestamp:
+                                        self._integratedValue = (
+                                            dataNoCenter
+                                            * (timestamp - self._lastTimestamp)
+                                            * 1e-6
+                                            - self._integratedCenter
+                                        )
+                                    else:
+                                        self._integratedValue += (
+                                            dataNoCenter
+                                            * (
+                                                (1 << 32)
+                                                - self._lastTimestamp
+                                                + timestamp
+                                            )
+                                            * 1e-6
+                                            - self._integratedCenter
+                                        )
 
                             self._count += 1
                             self._lastValue = data
@@ -401,13 +484,20 @@ class SPI:
                             # no data from the sensor just clear the last value
                             self._lastValue = 0
 
-                        off += self._xferSize
+                        self._lastTimestamp = timestamp
 
-
-    def initAccumulator(self, period: float, cmd: int, xferSize: int,
-                              validMask: int, validValue: int,
-                              dataShift: int, dataSize: int,
-                              isSigned: bool, bigEndian: bool) -> None:
+    def initAccumulator(
+        self,
+        period: float,
+        cmd: int,
+        xferSize: int,
+        validMask: int,
+        validValue: int,
+        dataShift: int,
+        dataSize: int,
+        isSigned: bool,
+        bigEndian: bool,
+    ) -> None:
         """Initialize the accumulator.
 
         :param period: Time between reads
@@ -423,21 +513,29 @@ class SPI:
         self.initAuto(xferSize * 2048)
 
         if bigEndian:
-            cmdBytes = cmd.to_bytes(4, 'big')
+            cmdBytes = cmd.to_bytes(4, "big")
         else:
-            cmdBytes = cmd.to_bytes(4, 'little')
+            cmdBytes = cmd.to_bytes(4, "little")
 
         self.setAutoTransmitData(cmdBytes, xferSize - 4)
         self.startAutoRate(period)
 
-        self.accum = self._Accumulator(self.port, xferSize, validMask, validValue,
-                                      dataShift, dataSize, isSigned, bigEndian)
+        self.accum = self._Accumulator(
+            self.port,
+            xferSize,
+            validMask,
+            validValue,
+            dataShift,
+            dataSize,
+            isSigned,
+            bigEndian,
+        )
         self.accum._notifier.startPeriodic(period * 1024)
 
     def freeAccumulator(self) -> None:
         """Frees the accumulator."""
         if self.accum:
-            self.accum.free()
+            self.accum.close()
             self.accum = None
         self.freeAuto()
 
@@ -449,6 +547,8 @@ class SPI:
             self.accum._value = 0
             self.accum._count = 0
             self.accum._lastValue = 0
+            self.accum._lastTimestamp = 0
+            self.accum._integratedValue = 0.0
 
     def setAccumulatorCenter(self, center: int) -> None:
         """Set the center value of the accumulator.
@@ -529,3 +629,47 @@ class SPI:
         with self.accum._mutex:
             self.accum._update()
             return AccumulatorResult(self.accum._value, self.accum._count)
+
+    def setAccumulatorIntegratedCenter(self, center: float) -> None:
+        """Set the center value of the accumulator integrator.
+
+        The center value is subtracted from each value*dt before it is added to the
+        integrated value. This is used for the center value of devices like gyros
+        and accelerometers to take the device offset into account when integrating.
+
+        """
+        if self.accum is None:
+            return
+
+        with self.accum._mutex:
+            self.accum._integratedCenter = center
+
+    def getAccumulatorIntegratedValue(self) -> float:
+        """Read the integrated value.  This is the sum of (each value * time between
+        values).
+
+        :returns: The integrated value accumulated since the last Reset().
+        """
+        if self.accum is None:
+            return
+
+        with self.accum._mutex:
+            self.accum._update()
+            return self.accum._integratedValue
+
+    def getAccumulatorIntegratedAverage(self) -> float:
+        """Read the average of the integrated value.  This is the sum of (each value
+        times the time between values), divided by the count.
+
+        :returns: The average of the integrated value accumulated since the last Reset().
+        """
+        if self.accum is None:
+            return 0.0
+
+        with self.accum._mutex:
+            self.accum._update()
+            if self.accum._count <= 1:
+                return 0.0
+
+            # count-1 due to not integrating the first value received
+            return self.accum._integratedValue / (self.accum._count - 1)
