@@ -1,10 +1,10 @@
 # validated: 2017-12-09 EN f9bece2ffbf7 edu/wpi/first/wpilibj/Talon.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
@@ -12,16 +12,16 @@ from .pwmspeedcontroller import PWMSpeedController
 
 __all__ = ["Talon"]
 
+
 class Talon(PWMSpeedController):
     """
         Cross the Road Electronics (CTRE) Talon and Talon SR Speed Controller via PWM
     """
 
-    def __init__(self, channel):
+    def __init__(self, channel: int) -> None:
         """Constructor for a Talon (original or Talon SR)
 
         :param channel: The PWM channel that the Talon is attached to. 0-9 are on-board, 10-19 are on the MXP port
-        :type  channel: int
 
         .. note ::
 
@@ -44,6 +44,5 @@ class Talon(PWMSpeedController):
         self.setSpeed(0.0)
         self.setZeroLatch()
 
-        hal.report(hal.UsageReporting.kResourceType_Talon,
-                   self.getChannel())
+        hal.report(hal.UsageReporting.kResourceType_Talon, self.getChannel())
         self.setName("Talon", self.getChannel())
