@@ -1,10 +1,10 @@
-# validated: 2017-12-02 EN 614093c0c4fc edu/wpi/first/wpilibj/Timer.java
-#----------------------------------------------------------------------------
+# validated: 2018-11-17 EN 89d15f061b33 edu/wpi/first/wpilibj/Timer.java
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import threading
 
@@ -12,6 +12,7 @@ import hal
 
 __all__ = ["Timer"]
 
+
 class Timer:
     """
         Provides time-related functionality for the robot
@@ -20,19 +21,18 @@ class Timer:
                   the :mod:`time` module in the standard library. This will
                   make it easier for your code to work properly in simulation. 
     """
-    
+
     @staticmethod
-    def getFPGATimestamp():
+    def getFPGATimestamp() -> float:
         """Return the system clock time in seconds. Return the time from the
         FPGA hardware clock in seconds since the FPGA started.
 
         :returns: Robot running time in seconds.
-        :rtype: float
         """
         return hal.getFPGATime() / 1000000.0
 
     @staticmethod
-    def getMatchTime():
+    def getMatchTime() -> float:
         """Return the approximate match time.
         The FMS does not currently send the official match time to the robots,
         but does send an approximate match time. The value will count down
@@ -46,13 +46,13 @@ class Timer:
         on the field.
 
         :returns: Time remaining in current match period (auto or teleop) in seconds
-        :rtype: float
         """
         from .driverstation import DriverStation
+
         return DriverStation.getInstance().getMatchTime()
 
     @staticmethod
-    def delay(seconds):
+    def delay(seconds: float) -> None:
         """Pause the thread for a specified time. Pause the execution of the
         thread for a specified period of time given in seconds. Motors will
         continue to run at their last assigned values, and sensors will
@@ -60,8 +60,7 @@ class Timer:
         until the wait time is expired.
 
         :param seconds: Length of time to pause
-        :type seconds: float
-        
+
         .. warning:: If you're tempted to use this function for autonomous
                      mode to time transitions between actions, don't do it!
                      
@@ -72,35 +71,35 @@ class Timer:
         """
         hal.sleep(seconds)
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.mutex = threading.RLock()
         self.startTime = self.getMsClock()
         self.accumulatedTime = 0.0
         self.running = False
 
-    def getMsClock(self):
+    def getMsClock(self) -> int:
         """
             :returns: the system clock time in milliseconds.
-            :rtype: int
         """
         return hal.getFPGATime() / 1000
 
-    def get(self):
+    def get(self) -> float:
         """Get the current time from the timer. If the clock is running it is
         derived from the current system clock the start time stored in the
         timer class. If the clock is not running, then return the time when
         it was last stopped.
 
         :returns: Current time value for this timer in seconds
-        :rtype: float
         """
         with self.mutex:
             if self.running:
-                return ((self.getMsClock() - self.startTime) + self.accumulatedTime) / 1000.0
+                return (
+                    (self.getMsClock() - self.startTime) + self.accumulatedTime
+                ) / 1000.0
             else:
                 return self.accumulatedTime
 
-    def reset(self):
+    def reset(self) -> None:
         """Reset the timer by setting the time to 0.
         Make the timer startTime the current time so new requests will be
         relative now.
@@ -109,7 +108,7 @@ class Timer:
             self.accumulatedTime = 0.0
             self.startTime = self.getMsClock()
 
-    def start(self):
+    def start(self) -> None:
         """Start the timer running.
         Just set the running flag to true indicating that all time requests
         should be relative to the system clock.
@@ -118,7 +117,7 @@ class Timer:
             self.startTime = self.getMsClock()
             self.running = True
 
-    def stop(self):
+    def stop(self) -> None:
         """Stop the timer.
         This computes the time as of now and clears the running flag, causing
         all subsequent time requests to be read from the accumulated time
@@ -128,22 +127,21 @@ class Timer:
             temp = self.get()
             self.accumulatedTime = temp
             self.running = False
-            
-    def hasPeriodPassed(self, period):
+
+    def hasPeriodPassed(self, period: float) -> bool:
         """Check if the period specified has passed and if it has, advance the start
         time by that period. This is useful to decide if it's time to do periodic
         work without drifting later by the time it took to get around to checking.
  
         :param period: The period to check for (in seconds).
         :returns: If the period has passed.
-        :rtype: bool
         """
-        
+
         with self.mutex:
             if self.get() > period:
                 # Advance the start time by the period
                 # Don't set it to the current time... we want to avoid drift
-                self.startTime += (period * 1000)
+                self.startTime += period * 1000
                 return True
-            
+
             return False
