@@ -1,15 +1,16 @@
 # validated: 2017-12-27 TW 8b7aa61091df edu/wpi/first/wpilibj/Utility.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2016. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
 __all__ = ["Utility"]
 
+
 class Utility:
     """Contains global utility functions
 
@@ -18,19 +19,18 @@ class Utility:
     """
 
     @staticmethod
-    def getFPGAVersion():
+    def getFPGAVersion() -> int:
         """Return the FPGA Version number.
 
         .. deprecated:: 2018.0.0
             Use :meth:`.RobotController.getFPGAVersion` instead
 
         :returns: FPGA Version number.
-        :rtype: int
         """
         return hal.getFPGAVersion()
 
     @staticmethod
-    def getFPGARevision():
+    def getFPGARevision() -> int:
         """Return the FPGA Revision number. The format of the revision is 3
         numbers.  The 12 most significant bits are the Major Revision. the
         next 8 bits are the Minor Revision. The 12 least significant bits
@@ -40,30 +40,27 @@ class Utility:
             Use :meth:`.RobotController.getFPGARevision` instead
 
         :returns: FPGA Revision number.
-        :rtype: int
         """
         return hal.getFPGARevision()
 
     @staticmethod
-    def getFPGATime():
+    def getFPGATime() -> int:
         """Read the microsecond timer from the FPGA.
 
         .. deprecated:: 2018.0.0
             Use :meth:`.RobotController.getFPGATime` instead
 
         :returns: The current time in microseconds according to the FPGA.
-        :rtype: int
         """
         return hal.getFPGATime()
 
     @staticmethod
-    def getUserButton():
+    def getUserButton() -> bool:
         """Get the state of the "USER" button on the roboRIO.
 
         .. deprecated:: 2018.0.0
             Use :meth:`.RobotController.getUserButton` instead
 
         :returns: True if the button is currently pressed down
-        :rtype: bool
         """
         return hal.getFPGAButton()
