@@ -1,10 +1,10 @@
 # validated: 2017-12-15 EN f9bece2ffbf7 edu/wpi/first/wpilibj/Victor.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
@@ -12,6 +12,7 @@ from .pwmspeedcontroller import PWMSpeedController
 
 __all__ = ["Victor"]
 
+
 class Victor(PWMSpeedController):
     """
         VEX Robotics Victor 888 Speed Controller via PWM
@@ -39,12 +40,11 @@ class Victor(PWMSpeedController):
         
         .. not_implemented: initVictor
     """
-    
-    def __init__(self, channel):
+
+    def __init__(self, channel: int) -> None:
         """Constructor.
 
         :param channel: The PWM channel that the Victor is attached to. 0-9 are on-board, 10-19 are on the MXP port
-        :type  channel: int
         """
         super().__init__(channel)
         self.setBounds(2.027, 1.525, 1.507, 1.49, 1.026)
@@ -52,6 +52,5 @@ class Victor(PWMSpeedController):
         self.setSpeed(0)
         self.setZeroLatch()
 
-        hal.report(hal.UsageReporting.kResourceType_Victor,
-                   self.getChannel())
-        self.setName('Victor', self.getChannel())
+        hal.report(hal.UsageReporting.kResourceType_Victor, self.getChannel())
+        self.setName("Victor", self.getChannel())
