@@ -1,10 +1,10 @@
 # validated: 2017-12-15 EN f9bece2ffbf7 edu/wpi/first/wpilibj/VictorSP.java
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 
@@ -12,16 +12,16 @@ from .pwmspeedcontroller import PWMSpeedController
 
 __all__ = ["VictorSP"]
 
+
 class VictorSP(PWMSpeedController):
     """
         VEX Robotics Victor SP Speed Controller via PWM
     """
 
-    def __init__(self, channel):
+    def __init__(self, channel: int) -> None:
         """Constructor.
 
         :param channel: The PWM channel that the VictorSP is attached to. 0-9 are on-board, 10-19 are on the MXP port.
-        :type  channel: int
 
         .. note ::
 
@@ -39,11 +39,10 @@ class VictorSP(PWMSpeedController):
             - 0.997ms = full "reverse"
         """
         super().__init__(channel)
-        self.setBounds(2.004, 1.52, 1.50, 1.48, .997)
+        self.setBounds(2.004, 1.52, 1.50, 1.48, 0.997)
         self.setPeriodMultiplier(self.PeriodMultiplier.k1X)
         self.setSpeed(0)
         self.setZeroLatch()
 
-        hal.report(hal.UsageReporting.kResourceType_VictorSP,
-                   self.getChannel())
+        hal.report(hal.UsageReporting.kResourceType_VictorSP, self.getChannel())
         self.setName("VictorSP", self.getChannel())
